from PyFRS import fs
import numpy as np
import matplotlib.pyplot as plt

if __name__ == '__main__':
    t = np.geomspace(1.e3, 1.e7, 300)
    nu = np.empty(t.shape)
    nu[:] = 1.e9
    Z = {'z': 0.1,  # Redshift
         'k': 0,  # Medium density profile index
         'SSC': 'No',  # SSC
         'XIC': 'No',  # EIC
         'E0': 1.e53,  # Isotropic-equivalent energy in erg
         'Gamma0': 2.,  # Initial bulk Lorentz factor
         'theta_j': 5.,  # Jet opening angle
         'theta_obs': 2.8,  # Viewing angle in deg
         'n18': 1.,  # CNM density at 10^18cm
         'p': 2.2,  # Electron energy distribution index
         'epsilon_e': 0.1,  # Fraction of the shock energy into electrons
         'epsilon_B': 0.01}
    Fnu = fs.FS_flux(t, nu, **Z)
    plt.plot(np.log10(t), np.log10(Fnu))
    fig, ax = plt.subplots(1, 1)
    ax.plot(t, Fnu)
    ax.set(xscale='log', xlabel=r'$t$ (s)',
           yscale='log', ylabel=r'$F_\nu$[$10^{9}$ Hz] (mJy)')

    fig.tight_layout()
    print("Saving figure lc.png")
    fig.savefig("lc1.png")
    plt.close(fig)




#================================================
#==============Reverse shock=====================

#-----------Scientific Library-------
import math
import numpy as np
import scipy.optimize

#-----------My Library---------
#Credit: Weihua Lei
from PyFRS import cgs,astro,grb

# try:
#     from . import cgs,astro,grb
# except ImportError:
#     import cgs,astro,grb


#def RS_flux(self):
def RS_flux(time_obs,nu_obs,**Z):
    log_obs_time = np.log10(time_obs)

#-------Input parameters: independent of model
#        zi= z
#        D28=astro.LUD(zi)/1.e28

#-------Unpack parameters
    for key,value in Z.items():
        if key == 'z':              # Redshift
            z = value
        elif key == 'k':            # Medium density profile index
            k = value
        #elif key == 'SSC':          # SSC
        #    if value == 1:
        #        SSC = 'Yes'
        #    else:
        #        SSC = 'No'
        #elif key == 'XIC':          # EIC
        #    if value == 1:
        #        XIC = 'Yes'
        #    else:
        #        XIC = 'No'
        elif key == 'E0':           # Isotropic-equivalent energy in 10^52 erg
            E52 = value/1.0e52
        elif key == 'Gamma0':       # Initial bulk Lorentz factor
            Gm0 = value
        elif key == 'theta_j':      # Jet opening angle
            theta_j = value
        elif key == 'theta_obs':    # Viewing angle in deg
            theta_obs = value
        elif key == 'n18':          # CNM density at 10^18cm
            n18 = value
        elif key == 'p':            # Electron energy distribution index
            p = value
        elif key == 'epsilon_e':    # Fraction of the shock energy into electrons
            epsilon_e = value
        elif key == 'epsilon_B':    # Fraction of the shock energy in magnetic field
            epsilon_B = value




#-------Input for jet
    zi=z
    D28=astro.LUD(zi)/1.e28

    n1=n18
    pp=p


    mytime=1.*cgs.day
    


    # Band={'X-ray': 1e18,'Optical': 1e15,'Radio': 1e9, 'GeV': 2.4e23}
#        Time={'input':tday, 'adjust':10.0**lgtday}
#    Dynmodel={'analytical': 1,'differential': 2}
#    with_SSC={'Yes':1, 'No':0}
#    with_jetbreak={'Yes':1, 'No':0}
    Tunits = {'Second': 1.0, 'Day': cgs.day}
    Nuunits= {'Hz': 1.0, 'keV': cgs.keV2Hz}

#    dynamic_model='differential'
    # Band_mode='X-ray'
#        v=Band[Band_mode]
    vlist=list(dict.fromkeys(list(nu_obs)))
    Funit=1.
    time_unit='Second'
    nu_unit='Hz'
#    jet_break='Yes'
#    inj_model='No'
#    Smooth='No'
    RS_Correction='No'  # Correct for the RS
    xi_s=1/3.**0.5  #crrection for radial spreading of RS
    xi_gm34=7/4.0 # correction for gamma34 in RS

#-------Initialize the time, gamma, Fv arries in Log space
    inum=500

    t=np.logspace(-4., 3.0,inum)
    to=np.logspace(-4., 3.0,inum)  # t_observe

    Rt=np.logspace(14.,21.,inum)
    Gmt=np.logspace(2.,-5.,inum)

    para=np.zeros((inum,19))

    lgt=np.logspace(-4., 3.0,inum)
    lgFv=np.logspace(1.,6.,inum)*np.ones((len(vlist),inum))
#    lgFv_SSC=np.logspace(1.,6.,inum)*np.ones((len(vlist),inum))

#    lgFv=np.logspace(1.,6.,inum)
#        lgFv_SSC=np.logspace(1.,6.,inum)

#    lgNu=np.logspace(7.,30.,inumv)
#    lgFnu=np.logspace(1.,6.,inumv)
#        lgFnu_SSC=np.logspace(1.,6.,inumv)

#-------Initialize critical parameters
#    lgNua=np.logspace(-4.,3.0,inump)
#    lgNum=np.logspace(-4.,3.0,inump)
#    lgNuc=np.logspace(-4.,3.0,inump)
#    lgFluxa=np.logspace(-4.,3.0,inump)
#    lgFluxm=np.logspace(-4.,3.0,inump)
#    lgFluxc=np.logspace(-4.,3.0,inump)

#    lgtdec=np.logspace(-4.,3.0,inump)
#    lgFluxdec=np.logspace(-4.,3.0,inump)
#         lgtSed=np.logspace(-4.,3.0,inump)
#         lgFluxSed=np.logspace(-4.,3.0,inump)


#-------start calculation
#    Gm0= Gamma0
    Gmt[1]=Gm0

    R0=Rt[1]
    Eiso=1.e52 *E52
#    t90=T90
    t[0]=0.
    t[1]=R0/(2.*Gm0**2.0*cgs.c*cgs.day)
    to[0]=0.
    to[1]=t[1]

    #if ( lgtday<math.log(t[1],10)):
    #     lgtday=math.log(t[1],10)

#-------jet, LoS, view angle
    #theta_obs=theta_obs
    thetaj=theta_j/cgs.deg
    thetaobs=theta_obs/cgs.deg
    fb=(1.-math.cos(thetaj))*2.*cgs.pi/(4.*cgs.pi)

    Gm4=Gm0
    Gm3x=Gm0
    Gm3=Gm0-1.e-4
    #Delta0=t90*cgs.c/(1.+zi)

    An0=n18 *1.e18**k
    Ne0=Eiso/(Gm0*cgs.mp*cgs.c**2.)
    xi=0.

    Delta0=R0/(3**0.5 * Gm0**2.)
    Delta=Delta0

    #spreading radius
    Rs=3**0.5 * Gm0**2. *Delta0
    beta4=math.sqrt(1.-1/Gm0**2.0)


    fview=1.
    Deltax=Rt[1]/(3**0.5 * Gm0**2.)  

    for i in range(1,inum):
        ni= n18*(Rt[i]/1.e18)**(- k)
        n1=ni

        Ri=Rt[i]
        dR=Ri-Rt[i-1]
        t[i]=Ri/(2.*Gm3**2.0 *cgs.c) *(1.+zi)
        ti=t[i]

        # check shell spreading
#        if (Ri<Rs):
#            Delta=Delta0
#        else:
#            Delta=Ri/(3**0.5 * Gm0**2.)

        Delta=Ri/(3**0.5 * Gm0**2.)   

        # check RS crossing
        if (xi< Deltax): #before crossing
            n4=Eiso/(Gm0*cgs.mp*cgs.c**2. *4.*cgs.pi*Ri**2. *Gm0*Delta)
            fn41=n4/n1
            beta3=math.sqrt(1.-1/Gm3**2.0)
            #Gm34=Gm0*Gm3*(1.-beta3*beta4)
            #Gm34=(Gm0/Gm3+Gm3/Gm0)/2.
            #Gm34=0.5*Gm3**2./fn41+ 1.
            Gm34=((Gm3**2.-1.)/fn41+ 1.)**0.5
            #print("Gm3=,Gm34=",Gm3,Gm34)
            gmcat3 = (4.*Gm34+1.)/(3.*Gm34)
            n43=(gmcat3-1.)/(gmcat3*Gm34+1.)
            n3=n4/n43
#            dx=dR/(Gm0*fn41**0.5 * (1.-1/4.))
            dx=dR/(Gm0*fn41**0.5 * (1.-Gm0/Gm3 * n43))

            xi=xi+dx
            xp=Gm0/Gm3 * n43* xi
            Ne3=n3*4.*cgs.pi*Ri**2.* Gm3*xp
            e3=(Gm34-1.)*n3*cgs.mp*cgs.c**2.
            Bc=(8.*cgs.pi *e3 * epsilon_B)**0.5

#-----------Max Lorentz factor-------
            gm_Max = grb.gamma_Max(Bc)
#-----------Minimum Lorentz factor-------
            gm_m=grb.gamma_m2(epsilon_e,Gm34,pp,gm_Max)
#-----------Cooling Lorentez factor
            gm_c=grb.gamma_c(Bc,Gm3,ti*fview,zi)

            #parameters at crossing
            e3x=e3
            n3x=n3
            n1x=n1
            fn41x=fn41
            Bcx=Bc
            txo=ti
            Rx=Ri
            Gm3x=Gm3
            Gm34x=Gm34
            gm_mx=gm_m
            gm_cx=gm_c
            Deltax=Delta

            #Gm3=(fn41*(Gm34**2.-1.) +1.)**0.5
            #Gm3=Gm0-1.e-4
            Gm3=grb.Gammat(Gm4,ni, Eiso, Ri,R0)

            #print("Gm34=, Gm3=, Delta=, Delta0=, xi=",Gm34,Gm3,Delta,Delta0,fn41**0.5/Gm0)

        else: #after crossing
            Ne3=Ne0

            gthick=(7-2.*k)/2.
            gthin=2-k/2.
            g= ((Gm34-1.)*gthick+gthin)/Gm34
            Gm3=Gm3x*(Ri/Rx)**(-g)
            n3=n3x*(Ri/Rx)**(-2.*g)

            if (Gm3 <=1.):
                Gm3=1.+1.e-6

            gthick_e=(26-4.*k)/3.
            gthin_e=4.-k
            g_e= ((Gm34-1.)*gthick_e+gthin_e)/Gm34
            e3=e3x*(Ri/Rx)**(-g_e)

            Bc=(8.*cgs.pi *e3 * epsilon_B)**0.5

            gm_Max = grb.gamma_Max(Bc)
            gm_m=gm_mx*(Ri/Rx)**(-g_e+2*g)
            gm_c=gm_cx*(Ri/Rx)**(-g_e+2*g)

            Pvmx=grb.Pvmax(Gm3x,Bcx)

            #print("### RS:: tx,gm_cx0,gm_mx0,Rx,n1x0,n3x,e3x,Bcx,fn41x,Gm341x:", txo,gm_cx,gm_mx,Rx,n1x,n3x,e3x,Bcx,fn41x,Gm34x)



        #print("Gmma3=",Gm3x)
        theta_view=grb.theta_off(Gm3,thetaobs,thetaj)
        fview=grb.aoff(Gm3,theta_view)


        to[i]=ti/fview  #off-viewer's time
        lgt[i]=math.log(ti/Tunits[ time_unit],10)
        ei=e3

#-----------v_Max, v_m, v_c, v_a----------
        vMax = grb.nu_gme(Gm3,gm_Max, Bc,zi)
        vm=grb.nu_gme(Gm3,gm_m, Bc,zi)
        vc=grb.nu_gme(Gm3,gm_c, Bc,zi)

        dL=Ne3/(4.*cgs.pi*Rt[i]**2. *n3)
        va=grb.nu_aB(Bc,pp,n3/(4.*Gm3),vm,vc,Gm3,dL*Gm3,zi)

        gm_a=(va*(2.*cgs.pi *cgs.me *cgs.c)*(1.+zi)/(Gm3 *cgs.qe *Bc))**0.5
#-----------Y-parameter for SSC without K-N correction
        UB = Bc**2. /(8.*cgs.pi)
        Uph=1.


#-----------t_min, t_col and t_colmin, t_jet
        # if (i>1):
        #     if ((v - vm) * (v - vm0) <= 0.0 ):
        #         #print("### RS:: time for vm cross:", Rt[i-1], ti/Tunits[ time_unit])
        #         t_min=ti/Tunits[time_unit]
        #     if ((v - vc) * (v - vc0) <= 0.0 ):
        #         #print("### RS:: time for vc cross:", Rt[i-1], ti/Tunits[ time_unit])
        #         t_col=ti/Tunits[time_unit]
        #     if ((vc - vm) * (vc0 - vm0) <= 0.0 ):
        #         #print("### RS:: time for vc=vm:", Rt[i-1], ti/Tunits[ time_unit])
        #         t_colmin=ti/Tunits[time_unit]
#-----------replace vm0 and vc0 for next loop
        vm0=vm
        vc0=vc
            

#        Rdec=(3.*Eiso/(Gm0**2* cgs.c**2 *4.*cgs.pi*n*cgs.mp))**(1/3.)



#-----------Pvm and Fvm
        Pvm=grb.Pvmax(Gm3,Bc)


#-----------Corrections on flux due to jet break effect
#            Fvm=grb.Fvmax(Pvm,n1,Ri,D28,zi)*fj*fDN /cgs.uJy

        for l,v in enumerate(vlist):
            Fvm=(1+zi)*Ne3*Pvm/(4.*cgs.pi*(D28*1e28)**2.) /cgs.uJy

#            Fvm_IC=1.
#-----------Corrections on off-axis jet for flux, see Beniamini et al. 2023
            fviewF=grb.FvOff(Gm3,thetaobs,thetaj)
            #fviewF=fview**3.

            if (t[i]<= txo):
                Fvti=fviewF *grb.Fv_sbpl(v/fview,va,vm,vc,Fvm,pp)  *Funit
            else:
                Fvti=fviewF *grb.Fv_RS(v/fview,va,vm,vc,Fvm,pp)  *Funit

#            if (v/fview>=vMax):
#                if (v/fview/vMax > 300.):
#                    Fvti=Fvti*np.exp(-300.+1.)
#                else:
#                    Fvti=Fvti*np.exp(-v/fview/vMax+1.)
#
            lgFv[l][i]=np.log10(Fvti)
#            lgFv[i]=np.log10(Fvti)
#            Fvt[i,0]=lgt[i]
#            Fvt[i,1]=lgFv[i]


#---------------Table for critical time
#         if (( t_dec <= to[i]/Tunits[ time_unit]) and ( t_dec > to[i-1]/Tunits[ time_unit])):
#             for j in range(0,inump):
#                  lgtdec[j]=np.log10( t_dec)
#                  lgFluxdec[j]=lgFv[i]-j/2.
# #                    lgFluxdec_n[j]=np.log10(fview_n**3. *Fvm_n)-j/2.
                
            
# #---------------Spectrum----------------------------
# #            mytime=Time[ Time_mode]
# #            if( (to[i]/cgs.day) >= mytime) and ( (to[i-1]/cgs.day) < mytime):
#         if( (t[i]/cgs.day) >= mytime) and ( (t[i-1]/cgs.day) < mytime):
#              nu_a=va/Nuunits[ nu_unit] *fview
#              nu_min=vm/Nuunits[ nu_unit] *fview
#              nu_cool=vc/Nuunits[ nu_unit] *fview

# #---------------Spectral index
#             dv = 1.05
#             Fvtip = fviewF *grb.Fv_sbpl(dv* v/fview,va,vm,vc,Fvm,pp)  *Funit
#             dF = Fvtip/Fvti
#             lgdF = math.log(dF,10)
#             lgdv = math.log(dv,10)
#             print("###[Spec Index]:",lgdF/lgdv)

# #---------------Table for critical syncrotron frequecies
#             Fvva=fviewF *grb.Fv_sbpl(va,va,vm,vc,Fvm,pp)
#             Fvvm=fviewF *grb.Fv_sbpl(vm,va,vm,vc,Fvm,pp)
#             Fvvc=fviewF *grb.Fv_sbpl(vc,va,vm,vc,Fvm,pp)
#             for j in range(0,inump):
#                  lgNua[j]=np.log10( nu_a)
#                  lgNum[j]=np.log10( nu_min)
#                  lgNuc[j]=np.log10( nu_cool)
#                  lgFluxa[j]=np.log10(Fvva)-j/2.
#                  lgFluxm[j]=np.log10(Fvvm)-j/2.
#                  lgFluxc[j]=np.log10(Fvvc)-j/2.

#             for j in range(1, inumv):
#                 vj=lgNu[j]
#                 Fvtj=fviewF *grb.Fv_sbpl(vj/fview,va,vm,vc,Fvm,pp)
#                 if (vj/fview>=vMax):
#                     if (vj/fview/vMax > 300.):
#                         Fvtj=Fvtj*np.exp(-300.+1.)
#                     else:
#                         Fvtj=Fvtj*np.exp(-vj/fview/vMax+1.)
#                 lgNu[j]=np.log10(vj/Nuunits[ nu_unit])
#                 lgFnu[j]=np.log10(Fvtj)





#---------------Parameters-----------
        para[i,0]=ti/Tunits[ time_unit]
        para[i,1]=Ri
        para[i,2]=Gm3
        para[i,3]=beta3
        para[i,4]=n3
        para[i,5]=ei
        para[i,6]=Bc
        para[i,7]=Uph
        para[i,8]=UB
        para[i,9]=gm_a
        para[i,10]=gm_m
        para[i,11]=gm_c
        para[i,12]=gm_Max
        para[i,13]=va*fview
        para[i,14]=vm*fview
        para[i,15]=vc*fview
        para[i,16]=vMax*fview
        para[i,17]=Fvm
#            para[i,18]=Fvm_IC
        para[i,18]=Gm34


#-----------small correction
    Gmt[0]=Gmt[1]
    lgt[0]=lgt[2]
    lgFv[:,0]=lgFv[:,2]
#    lgFv[0]=lgFv[2]
    # lgNu[0]=lgNu[2]
    # lgFnu[0]=lgFnu[2]
    lgt[1]=lgt[2]
    lgFv[:,1]=lgFv[:,2]
    # lgNu[1]=lgNu[2]
    # lgFnu[1]=lgFnu[2]
    for j in range(0,19):
        para[0,j]=para[1,j]

#-------transfer data to global
    paralist=para
    #  lgLC=np.column_stack((lgt,lgFv))
    #  lgSpec=np.column_stack((lgNu,lgFnu))
#         lgLC_SSC=np.column_stack((lgt,lgFv_SSC))
#         lgSpec_SSC=np.column_stack((lgNu,lgFnu_SSC))

#        np.savetxt( file_dir+'Output/vat_n.txt',vat) #t, va,vm,vc
#        np.savetxt( file_dir+'Output/Coolingt_n.txt',Coolingt)
#        np.savetxt( file_dir+'Output/Bt_n.txt',Bt_n)
#        np.savetxt( file_dir+'Output/para_n.txt',para) #t, Fvm, Gamma, R, Bc, gamma_m, gamma_c
#    print("### RS:: txo1,gm_cx1,gm_mx0,Rx,n1x0,n3x,e3x,Bcx,fn41x,Gm341x:", txo,gm_cx,gm_mx,Rx,n1x,n3x,e3x,Bcx,fn41x,Gm341x)
#        np.savetxt('/gtap/data/t.txt',10**(lgt))

#================end of Reverse shock==================
    lgFv = lgFv-3.0   # convert unit to "mJy"


#--------Estimate the flux of obs_time (add by fsy)
    estimate_model_log_flux = np.zeros(len(log_obs_time))
    for i,t in enumerate(log_obs_time):
        for iline in range(len(vlist)):
            if nu_obs[i] == vlist[iline]:
                vline = iline
                break
        model_log_time = lgt
        estimate_model_log_flux[i] = np.interp(t,model_log_time,lgFv[vline],left=-99.0,right=99.0)
    
    return 10**estimate_model_log_flux




#==================================PyGRB Toolkit===--==============================
#  A Python tool for GRB model Fit
#Credit: Weihua Lei
#Institue for Particle Physics and Astrophysics, Huazhong University of Science and Technology (HUST), Wuhan, China
#Email: leiwh@hust.edu.cn
#Homepage: http://www.physics.unlv.edu/~leiwh/pygrb/
#
#  Description:
#Can be applied to GRB, TDE, etc.
#Models: forward shock, energy injection, two-component jet, SSC, IC by X-ray photons from internal shock (XIC)
#Created on Tue Aug 16 19:39:07 2011, University of Nevada Las Vegas (UNLV), USA
#Current version: 2014-11-25
#
#  Reference:
#-------afterglow dynamics and emission:
#1. A complete reference of the analytical synchrotron external shock models of gamma-ray bursts, He Gao, Wei-Hua Lei, Yuan-Chuan Zou, Xue-Feng Wu, Bing Zhang,2013, New Astronomy Review, 57, 141-190
#2. Overall Evolution of Jetted Gamma-Ray Burst Ejecta, Y. F. Huang, L. J. Gou, Z. G. Dai, T. Lu, 2000, ApJ, 543, 90-96
#-------central engine models:
#1. Hyperaccreting Black Hole as Gamma-Ray Burst Central Engine. I. Baryon Loading in Gamma-Ray Burst Jets, Wei-Hua Lei, Bing Zhang, En-Wei Liang, 2013, ApJ, 765, 125
#2. Giant X-Ray Bump in GRB 121027A: Evidence For Fall-Back Disk Accretion, Xue-Feng Wu, Shu-Jin Hou, Wei-Hua Lei, 2013, ApJ, 765, 36 
#
#
#  These papers used our code:
#QUASI-PERIODIC VARIATIONS IN X-RAY EMISSION AND LONG-TERM RADIO OBSERVATIONS: EVIDENCE FOR A TWO-COMPONENT JET IN Sw J1644+57, Jiu-Zhou Wang, Wei-Hua Lei, Ding-Xiong Wang, Yuan-Chuan Zou, Bing Zhang, He Gao, Chang-Yin Huang, 2014, ApJ, 788, 32
#
#==============================================================================

import math
from PyFRS import cgs,astro
import numpy as np
from scipy.special import kv
from scipy.integrate import quad

#Doppler ratio for off-beaming
#off-beaming angle
def theta_off(GM,thetaobs,thetaj):
    thetacone=1./GM
    if (thetaj>=thetacone):
        thetajet=thetaj
    else:
        thetajet=thetacone

    if (thetaobs <= thetajet):
        theta_view=0.
    else:
        theta_view=(thetaobs - thetajet)
    
    return theta_view


def aoff(GM,theta_view):
    beta=math.sqrt(1.-1./GM**2.0)
    return (1.-beta)/(1.-beta*math.cos(theta_view) )



#Off-beaming correction factor to Fv
def FvOff(GM,thetaobs,thetaj):
    thetacone=1./GM
    theta_view=theta_off(GM,thetaobs,thetaj)
    fview=aoff(GM,theta_view)    
    if (thetaj<=thetacone):
        if ((thetaobs>=0.) and (thetaobs<=-thetaj+thetacone)):
            fviewF=1.
        elif ((thetaobs>-thetaj+thetacone) and (thetaobs<=thetacone)):
            fviewF=1.-(thetaobs+thetaj-thetacone)/2./thetaj
        elif ((thetaobs>thetacone) and (thetaobs<2.*thetacone)):
            fviewF=fview**2. *fview*(1.+(GM* theta_view)**2.)/2.
        elif (thetaobs>=2.*thetacone):
            fviewF=fview**3.
    else:
        if ((thetaobs>=0.) and (thetaobs<=thetaj-thetacone)):
            fviewF=1.
        elif ((thetaobs>thetaj-thetacone) and (thetaobs<=thetaj)):
            fviewF=1.-GM*(thetaobs-thetaj+thetacone)/2.
        elif ((thetaobs>thetaj) and (thetaobs<2.*thetaj)):
            fviewF=fview**2. *fview*(1.+(GM* theta_view)**2.)/2.
        elif (thetaobs>=2.*thetaj):
            fviewF=fview**3. *(1.+(GM* thetaj)**2.)/2.
    return fviewF




#time for nu_m cross
def tm(eb,ee,E52,nu15):
    return 0.69*eb**(1/3.)*ee**(4/3.)*E52**(1/3.)*nu15**(-2/3.)

#time for nu_c cross    
def tc(eb,E52,n,nu15):
    return 7.3e-6*eb**(-3.)*n**(-2.)*E52**(-1.)*nu15**(-2.)

#band function
def NE(E,alpha,beta,E0):
    if (E<(alpha-beta)*E0):
        s=E**alpha *math.exp(-E/E0)
    else:
        s=((alpha-beta)*E0)**(alpha-beta)*E**beta*math.exp(beta-alpha)
    return s

#bulk Lorentz factor Gamma as a function of R
#valid only for deceraction phase, no energy injection, ISM case 
def Gammat2(Gamma0,R,R0):
    return Gamma0*(R/R0)**(-3/2.)
    
#bulk Lorentz factor Gamma as a function of R
#valid for both pre and after deceraction radius, no energy injection, ISM case 
def Gammat1(Gm0,n, Eiso, R,R0):
    eps=1.e-6
    A=4.*cgs.pi*R**3.*n*cgs.mp/3.
    B=Eiso/(Gm0*cgs.c**2.)-A
    C=Eiso/cgs.c**2.
    ABC=4.*A*C/B**2.
    if (ABC <eps):
        Gmt=Gm0
    else:
        Gmt=(-B+math.sqrt(B**2.* (1.+ABC)))/(2.*A)
    return Gmt

#bulk Lorentz factor Gamma as a function of R
#valid for both pre and after deceraction radius, no energy injection, ISM case
def Gammat(Gm0,n, Eiso, R,R0):
    eps=1.e-6
    A=4.*cgs.pi*R**3.*n*cgs.mp/3.
    B=Eiso/(Gm0*cgs.c**2.)
    C=Eiso/cgs.c**2.+A
    ABC=4.*A*C/B**2.
    if (ABC <eps):
        Gmt=Gm0+A/B
    else:
        Gmt=(-B+math.sqrt(B**2.* (1.+ABC)))/(2.*A)
    return Gmt
 
#Deceration radius   
def Rdec(Gm0,n, Eiso):
    return (3.*Eiso/(Gm0**2.* cgs.c**2. *2.*cgs.pi*n*cgs.mp))**(1/3.)

#Deceration time
def Tdec(Gm0,n,Eiso):
    return Rdec(Gm0,n,Eiso)/2./Gm0**2/cgs.c

#Sedov radius
def Rsed(n, Eiso):
    return (3.*Eiso/(cgs.c**2 *4.*cgs.pi*n*cgs.mp))**(1/3.)

#Comoving B, version 1: with input parameters: epsilon_B, Gamma, n 
def Bco(epsilon_B, Gamma0, n1):
    return math.sqrt(8.*cgs.pi*(4.*Gamma0+3.)*(Gamma0-1.)*cgs.mp*epsilon_B*n1)*cgs.c

#Comoving B, version 2: with input parameters: epsilon_B, e
def Bco2(epsilon_B, ei):
    return math.sqrt(8.*cgs.pi* epsilon_B* ei)

def n2_c(GM21,n1):
    gmcat = (4.*GM21+1.)/(3.*GM21)
    n2=(gmcat*GM21+1.)/(gmcat-1.)*n1
    return n2


def e2(GM21,n1):
    gmcat = (4.*GM21+1.)/(3.*GM21)
    if (GM21-1.) > 1.e-3:
#--------For Relativistic case        
#        gmcat=4./3.
#        n2n1=(gmcat*GM21+1.)/(gmcat-1.)
        GMm1 = GM21 -1.
    else:
#--------Newtonian case        
#        gmcat=5./3.
        beta=math.sqrt(1.-1./GM21**2.0)
        GMm1 = 0.5*(beta)**2
    n2n1=(gmcat*GM21+1.)/(gmcat-1.)
    ei= n2n1* GMm1 *n1 *cgs.mp *cgs.c**2
    return ei

#--------------------------------------------------------------------------
#---------------------------------gamma--------------------------------------
#--------------------------------------------------------------------------

#max electron Lorentz factor: gamma_m
def gamma_Max(Bc):
    zeta = 1.
    return math.sqrt(6.*cgs.pi*cgs.qe/(cgs.sigmaT* Bc* zeta))

#minimum electron Lorentz factor: gamma_m
def gamma_m(epsilon_e,GM,p):   
#---for p>2
    g_p=(p-2.)/(p-1.)
    npne=1.0
    gamma_ave= epsilon_e*  (GM-1.)*npne* cgs.mp/cgs.me  
    return g_p* gamma_ave

def gamma_m2(epsilon_e,GM,p, g_M):
#---for 1<p<2, p!=1, p!=2, include Newtonian case
    if p > 2.:
        if (GM-1.) > 1.e-3:
            gsm = gamma_m(epsilon_e,GM,p)
        else:
            beta=math.sqrt(1.-1./GM**2.0)
            g_p=(p - 2.)/(p - 1.)
            npne=1.0
            gamma_ave=epsilon_e* 0.5*(beta)**2 *npne* cgs.mp/cgs.me 
            gsm = g_p* gamma_ave
    elif 1.< p < 2.:
        pp = (2.-p)/(p-1.)
        npne = 1.
        mpme = cgs.mp/cgs.me
        GMm1 = GM -1.
        if (GM-1.) <= 1.e-3:
            beta=math.sqrt(1.-1./GM**2.0)
            GMm1 = 0.5*(beta)**2
        gsm = (epsilon_e *GMm1 *pp*mpme*npne/g_M**(2.-p) )**(1./(p-1.))
    return gsm

    
#cooling electron Lorentz factor: gamma_c    
def gamma_c(Bc,GM,t,z):
    return 6.*cgs.pi*cgs.me*cgs.c/(cgs.sigmaT* Bc**2* GM* t/(1+z))

#get nu in lab frame by given electron Lorentz factor 
def nu_gme(GM, gm_e, Bc, z):
    return GM* gm_e**2 *cgs.qe *Bc/(2.*cgs.pi *cgs.me *cgs.c)/(1.+z)
#    return GM* gm_e**2 *cgs.qe *Bc*3./(4.*cgs.pi *cgs.me *cgs.c)/(1.+z)

#get maximum frequence in Lab frame: nu_Max
def nu_Max(GM, z):
    return 3.* GM *cgs.qe**2/(2.*cgs.pi *cgs.me *cgs.c)/(1.+z)/(cgs.sigmaT)

def Fvmax_analy(epsilon_B, n, E52, D28):
    return 1.1e5*epsilon_B**(1/2.)*E52*n**(1/2.)*D28**(-2.)

def Pvmax(GM, Bc):
    #Pvm_old = cgs.me*cgs.c**2.0*cgs.sigmaT*GM*Bc/(3.*cgs.qe)
    phi_cr=1. #see Wijere & Galama 1999
    Pvm_new=3.**0.5 * phi_cr * cgs.qe**3. *GM*Bc/(cgs.me*cgs.c**2.) # average peak power
    return Pvm_new

def Fvmax(Pvm,n1,R,D28,z):
    Ne=4.*cgs.pi*R**3.*n1/3
    return (1+z)*Ne*Pvm/(4.*cgs.pi*(D28*1e28)**2.)
    
def Fvmax_IC(Fvm, n1, R):
    x0=0.4714 #sqrt(2)/3
    return cgs.sigmaT *n1 *R *Fvm* x0

def Fvmax_IC0(Fvm, n1, R):
    return cgs.sigmaT *n1 *R *Fvm/3.

def Fvmax3(m,epsilon_B,Gamma0, n1,D28):
    Ne=m/cgs.mp
    B=Bco(epsilon_B,Gamma0,n1)
    Pvm=Pvmax(Gamma0,B)
    return Ne*Pvm/(4.*cgs.pi*(D28*1e28)**2.)/cgs.uJy


#--------------------------------------------------------------------------
#---------------------------------nu--------------------------------------
#--------------------------------------------------------------------------
#time evolution of nu_c and nu_m
#valid only for deceration phase 
def nu_c(epsilon_B, n, E52, td):
    return 2.7e12* epsilon_B**(-3/2.)*E52**(-1/2.)*td**(-1/2.)/n

def nu_m(epsilon_B, epsilon_e, E52, td):
    return 5.7e14* epsilon_B**(1/2.)*epsilon_e**2.*E52**(1/2.)*td**(-3/2.)




#nu=GM* gm_e**2 *cgs.qe *Bc/(2.*cgs.pi *cgs.me *cgs.c)/(1.+z)
#the first version of nu_a
def nu_a1(epsilon_B,p,n1,vm,vc,Gamma0,R,z):
    B=Bco(epsilon_B,Gamma0,n1)
    gamma_nu=(2.*cgs.pi*cgs.me*cgs.c*(1.+z)/(Gamma0*cgs.qe*B))**0.5
    gamma_m= vm**0.5 *gamma_nu
    gamma_c= vc**0.5 *gamma_nu
    if (vm<vc):
        Cg1=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
        alpha=1.0e4*(8.4e6)**(p/2.)*Cg1*B**(p+2.)/2.*math.gamma((3.*p+2.)/12.)*math.gamma((3.*p+22.)/12.)*R/Gamma0
        tau_vm=alpha*(vm*(1.+z)/Gamma0)**(-(p+4.)/2.)
        tau_vc=alpha*(vc*(1.+z)/Gamma0)**(-(p+4.)/2.)
        if (tau_vm <1.):
            Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
            va=(136.*(p+2.)/(p+2./3.)*Cg*B**(2./3.)*gamma_m**(-(p+2./3.)) *R/Gamma0)**(3./5.) *Gamma0/(1.+z)
        else:
            if (tau_vc <1.):
                Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
#                va= alpha**(2./(p+4.))*Gamma0/(1.+z)
                pj=p
                va=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
            else:
                Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)*gamma_c
                pj=p+1.
                va=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
    else:
        Cg1=4.*Gamma0*n1*gamma_c
        alpha=1.0e4*8.4e6*Cg1*B**2.*math.gamma((3.*2.+2.)/12.)*math.gamma((3.*2.+22.)/12.)*R/Gamma0
        tau_vm=alpha*(vm*(1.+z)/Gamma0)**(-3.)
        tau_vc=alpha*(vc*(1.+z)/Gamma0)**(-3.)
        if (tau_vc <1.):
            Cg=4.*Gamma0*n1*gamma_c
            va=(136.*(2.+2.)/(2.+2./3.)*Cg*B**(2./3.)*gamma_c**(-(2.+2./3.))*R/Gamma0)**(3/5.)*Gamma0/(1.+z)
        else:
            if (tau_vm <1.):
                Cg=4.*Gamma0*n1*gamma_c
#                va= alpha**(1/3.)*Gamma0/(1.+z)
                pj=2.
                va=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
            else:
                Cg=4.*Gamma0*n1*gamma_m**(p-1.)*gamma_c
                pj=p+1.
                va=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
    return va
    
#the second version of nu_a
def nu_a2(epsilon_B,p,n1,vm,vc,Gamma0,R,z):
    B=Bco(epsilon_B,Gamma0,n1)
    gamma_nu=(2.*cgs.pi*cgs.me*cgs.c*(1.+z)/(Gamma0*cgs.qe*B))**0.5
    gamma_m= vm**0.5 *gamma_nu
    gamma_c= vc**0.5 *gamma_nu
    if (vm<vc):
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
        va1=(136.*(p+2.)/(p+2./3.)*Cg*B**(2./3.)*gamma_m**(-(p+2./3.)) *R/Gamma0)**(3./5.) *Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
        pj=p
        va2=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
        if (va1 <= va2):
            va=va1
        else:
            if (va2 <= va3):
                va=va2
            else:
                va=va3
    else:
        Cg=4.*Gamma0*n1*gamma_c
        va1=(136.*(2.+2.)/(2.+2./3.)*Cg*B**(2./3.)*gamma_c**(-(2.+2./3.))*R/Gamma0)**(3/5.)*Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*gamma_c
        pj=2.
        va2=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
        Cg=4.*Gamma0*n1*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
        if (va1 <= va2):
            va=va1
        else:
            if (va2 <= va3):
                va=va2
            else:
                va=va3

    return va

#the third version of nu_a
#nu_a is not smoothly connected    
def nu_a3(epsilon_B,p,n1,vm,vc,Gamma0,R,z):
    B=Bco(epsilon_B,Gamma0,n1)
    gamma_nu=(2.*cgs.pi*cgs.me*cgs.c*(1.+z)/(Gamma0*cgs.qe*B))**0.5
    gamma_m= vm**0.5 *gamma_nu
    gamma_c= vc**0.5 *gamma_nu
    if (vm<vc):
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
        va1=(136.*(p+2.)/(p+2./3.)*Cg*B**(2./3.)*gamma_m**(-(p+2./3.)) *R/Gamma0)**(3./5.) *Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
        pj=p
        va2=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
        if (va1 <= vm):
            va=va1
        else:
            if (va2 <= vc):
                va=va2
            else:
                va=va3
    else:
        Cg=4.*Gamma0*n1*gamma_c
        va1=(136.*(2.+2.)/(2.+2./3.)*Cg*B**(2./3.)*gamma_c**(-(2.+2./3.))*R/Gamma0)**(3/5.)*Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*gamma_c
        pj=2.
        va2=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
        Cg=4.*Gamma0*n1*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=(1.0e4*(8.4e6)**(pj/2.)*Cg*B**((pj+2.)/2.)*math.gamma((3.*pj+2.)/12.)*math.gamma((3.*pj+22.)/12.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
        if (va1 <= vc):
            va=va1
        else:
            if (va2 <= vm):
                va=va2
            else:
                va=va3
    return va

#the fourth of nu_a
#modified for smooth connection between transition
def nu_a(epsilon_B,p,n1,vm,vc,Gamma0,R,z):
    B=Bco(epsilon_B,Gamma0,n1)
    c0=(2.**(4/3.)*math.gamma(1/3.))**(-1.) * 4.*cgs.pi**2. *cgs.qe
    cnugm=2.*cgs.pi*cgs.me*cgs.c/cgs.qe
    gamma_nu=(cnugm*(1.+z)/B/Gamma0)**0.5
    gamma_m= vm**0.5 *gamma_nu
    gamma_c= vc**0.5 *gamma_nu
    if (vm<vc):
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
        pj=p
        va1=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-5./3.)*gamma_m**(-(pj+2./3.)) *R/Gamma0)**(3./5.) *Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)
        pj=p
        va2=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*(p-1.)*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)
        if (va1 <= vm):
            va=va1
        else:
            if (va2 <= vc):
                va=va2
            else:
                va=va3
    else:
        Cg=4.*Gamma0*n1*gamma_c
        pj=2.
        va1=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-5./3.)*gamma_c**(-(pj+2./3.))*R/Gamma0)**(3/5.)*Gamma0/(1.+z)
        Cg=4.*Gamma0*n1*gamma_c
        pj=2.
        va2=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
        Cg=4.*Gamma0*n1*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma0)**(2./(pj+4.))*Gamma0/(1.+z)    
        if (va1 <= vc):
            va=va1
        else:
            if (va2 <= vm):
                va=va2
            else:
                va=va3
    return va

#the fivth and current version of nu_a with given B
#modified for smooth connection between transition
def nu_aB(Bc,p,n1,vm,vc,Gamma,R,z):
    B=Bc
#    c0=(2.**(4/3.)*math.gamma(1/3.))**(-1.) * 4.*cgs.pi**2. *cgs.qe
    c0= 2.8071453296870878e-09
#    cnugm=2.*cgs.pi*cgs.me*cgs.c/cgs.qe
    cnugm=3.575013703788261e-07
    gamma_nu=math.sqrt(cnugm*(1.+z)/B/Gamma)
    gamma_m= math.sqrt(vm) *gamma_nu
    gamma_c= math.sqrt(vc) *gamma_nu
    if (vm<vc):
        Cg=4.*Gamma*n1*(p-1.)*gamma_m**(p-1.)
        pj=p
        va1=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-5./3.)*gamma_m**(-(pj+2./3.)) *R/Gamma)**(3./5.) *Gamma/(1.+z)
#        Cg=4.*Gamma*n1*(p-1.)*gamma_m**(p-1.)
#        pj=p
        va2=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma)**(2./(pj+4.))*Gamma/(1.+z)
        Cg=4.*Gamma*n1*(p-1.)*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma)**(2./(pj+4.))*Gamma/(1.+z)
        if (va1 <= vm):
            va=va1
        else:
            if (va2 <= vc):
                va=va2
            else:
                va=va3
    else:
        Cg=4.*Gamma*n1*gamma_c
        pj=2.
        va1=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-5./3.)*gamma_c**(-(pj+2./3.))*R/Gamma)**(3/5.)*Gamma/(1.+z)
#        Cg=4.*Gamma*n1*gamma_c
#        pj=2.
        va2=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma)**(2./(pj+4.))*Gamma/(1.+z)    
        Cg=4.*Gamma*n1*gamma_m**(p-1.)*gamma_c
        pj=p+1.
        va3=((c0/B)*(pj+2.)/(pj+2./3.)*Cg*(cnugm/B)**(-(pj+4.)/2.)*R/Gamma)**(2./(pj+4.))*Gamma/(1.+z)    
        if (va1 <= vc):
            va=va1
        else:
            if (va2 <= vm):
                va=va2
            else:
                va=va3
    return va

#--------------------------------------------------------------------------
#---------------------------------Syn--------------------------------------

#-------------------------Trail version of Fv---------------------
#no nu_a
def Fv_trail(v, vm, vc, vmax, Fvm, p):
    if (vc<=vm):
#------fast cooling-------
        if (v<=vc):
            Fnu=(v/vc)**(1/3.)* Fvm
        elif (v>vc) and (v<=vm):
            Fnu=(v/vc)**(-1/2.)*Fvm
        elif (v>vm) and (v<=vmax):
            Fnu=(vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm
        elif (v>vmax):
            Fnu=(vm/vc)**(-1/2.)*(vmax/vm)**(-p/2.)*Fvm * np.exp(1.-v/vmax)
    else:
#------slow cooling------
        if (v<=vm):
            Fnu=(v/vm)**(1/3.)* Fvm
        elif (v>vm) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc) and (v<=vmax):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
        elif (v>vmax):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(vmax/vc)**(-p/2.)*Fvm * np.exp(1.-v/vmax)
    return Fnu


#------------------------Simple braken power law---------------------------
#Fv in fast cooling case
def Fvfast_sbpl(v,va,vm,vc,Fvm,p):
    if (va<=vc):
        if (v<=va):
            Fnu=(va/vc)**(1/3.) *(v/va)**2. *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vc)**(1/3.)* Fvm
        elif (v>vc) and (v<=vm):
            Fnu=(v/vc)**(-1/2.)*Fvm
        elif (v>vm):
            Fnu=(vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm
    elif (va>vc) and (va<=vm):
        if (v<=vc):
            Fnu=(vc/va)**3. *(v/vc)**2. *Fvm
        elif (v>vc) and (v<=va):
            Fnu=(va/vc)**(-1/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vm):
            Fnu=(v/vc)**(-1/2.)*Fvm
        elif (v>vm):
            Fnu=(vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm    
    elif (va>vm):
        if (v<=vc):
            Fnu=(vc/va)**3. *(vm/va)**((p-1.)/2.) *(v/vc)**2. *Fvm
        elif (v>vc) and (v<=va):
            Fnu=(vm/vc)**(-1/2.) *(va/vm)**(-p/2.) *(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm
    return Fnu

#Fv in slow cooling case        
def Fvslow_sbpl(v,va,vm,vc,Fvm,p):
    if (va<=vm):
        if (v<=va):
            Fnu=(va/vm)**(1/3.)*(v/va)**2. *Fvm
        elif (v>va) and (v<=vm):
            Fnu=(v/vm)**(1/3.)* Fvm
        elif (v>vm) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vm) and (va<=vc):
        if (v<=vm):
            Fnu=(vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vc):
        if (v<=vm):
            Fnu=(vc/va)**0.5* (vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    return Fnu

def Fv_sbpl(v,va,vm,vc,Fvm,p):
    if (vc<=vm):
#        Fnu=Fvfast(v,va,vm,vc,Fvm,p)
        Fnu=Fvfast_sbpl(v,va,vm,vc,Fvm,p)
    else:
#        Fnu=Fvslow(v,va,vm,vc,Fvm,p)
        Fnu=Fvslow_sbpl(v,va,vm,vc,Fvm,p)
    return Fnu



def vFv_sbpl(v,va,vm,vc,Fvm,p):
    return v*Fv_sbpl(v,va,vm,vc,Fvm,p)



#------------------------Simple braken power law for RS---------------------------
#Fv in fast cooling case
def Fvfast_RS(v,va,vm,vc,Fvm,p):
    if (v>vc):
        Fnu=cgs.zero
#        Fnu=np.exp(-100)
    elif (va<=vc):
        if (v<=va):
            Fnu=(va/vc)**(1/3.) *(v/va)**2. *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vc)**(1/3.)* Fvm
        elif (v>vc) and (v<=vm):
            Fnu=(v/vc)**(-1/2.)*Fvm
        elif (v>vm):
            Fnu=(vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm
    elif (va>vc) and (va<=vm):
        if (v<=vc):
            Fnu=(vc/va)**3. *(v/vc)**2. *Fvm
        elif (v>vc) and (v<=va):
            Fnu=(va/vc)**(-1/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vm):
            Fnu=(v/vc)**(-1/2.)*Fvm
        elif (v>vm):
            Fnu=(vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm    
    elif (va>vm):
        if (v<=vc):
            Fnu=(vc/va)**3. *(vm/va)**((p-1.)/2.) *(v/vc)**2. *Fvm
        elif (v>vc) and (v<=va):
            Fnu=(vm/vc)**(-1/2.) *(va/vm)**(-p/2.) *(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm
    return Fnu

#Fv in slow cooling case        
def Fvslow_RS(v,va,vm,vc,Fvm,p):
    if (v>vc):
        Fnu=cgs.zero
#        Fnu=np.exp(-100)
    elif (va<=vm):
        if (v<=va):
            Fnu=(va/vm)**(1/3.)*(v/va)**2. *Fvm
        elif (v>va) and (v<=vm):
            Fnu=(v/vm)**(1/3.)* Fvm
        elif (v>vm) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
#    elif (v>vc):
#                Fnu=np.exp(-700)
#        Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vm) and (va<=vc):
        if (v<=vm):
            Fnu=(vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
#    elif (v>vc):
#        Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vc):
#        Fnu=np.exp(-100)
        if (v<=vm):
            Fnu=(vc/va)**0.5* (vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
#    elif (v>va):
#        Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    return Fnu


def Fv_RS(v,va,vm,vc,Fvm,p):
    if (vc<=vm):
        Fnu=cgs.zero
#        Fnu=Fvfast(v,va,vm,vc,Fvm,p)
#        Fnu=Fvfast_RS(v,va,vm,vc,Fvm,p)
    else:
#        Fnu=Fvslow(v,va,vm,vc,Fvm,p)
        Fnu=Fvslow_RS(v,va,vm,vc,Fvm,p)
    return Fnu



def vFv_RS(v,va,vm,vc,Fvm,p):
    return v*Fv_RS(v,va,vm,vc,Fvm,p)




#---------------------Starting update 1 for Fv---------------------------------    
#Current version:
#updates ver.1 for Fv, smooth for nu_a   
#modified for smooth connection between transition at nu_a
#Fv in fast cooling case
def Fvfast(v,va,vm,vc,Fvm,p):
    if (va<=vc):
        tauv = (v/va)**(-5./3.)
        tau = np.exp(-tauv)
        Sv = (va/vc)**(1/3.) *(v/va)**2. *Fvm
        Fnu_thick = Sv*(1.-tau)
        if (tauv <1.e-3):
            Fnu_thick=(v/vc)**(1/3.)* Fvm
        if (v<=va):
                Fnu = Fnu_thick
        elif (v>va) and (v<=vc):
            Fnu = Fnu_thick
        elif (v>vc) and (v<=vm):
            Fnu = Fnu_thick* (v/vc)**(-1/2.-1/3.)
        elif (v>vm):
            Fnu = Fnu_thick* (v/vc)**(-1/2.-1/3.)*(v/vm)**(-p/2.+1/2.)
    elif (va>vc) and (va<=vm):
        tauv = (v/va)**(-3.)
        tau = np.exp(-tauv)
        Sv = (va/vc)**(-1/2.) *(v/va)**2.5 *Fvm
        Fnu_thick = Sv*(1.-tau)
        if (tauv <1.e-3):
            Fnu_thick = (v/vc)**(-1/2.)*Fvm
    if (v<=vc):
        Fnu=Fnu_thick* (v/vc)**(-1/2.)
    elif (v>vc) and (v<=va):
        Fnu = Fnu_thick
    elif (v>va) and (v<=vm):
        Fnu=Fnu_thick
    elif (v>vm):
        Fnu=Fnu_thick* (v/vm)**(-p/2.+1/2.)   
    elif (va>vm):
        tauv = (v/va)**(-(p+5.)/2.)
        tau = np.exp(-tauv)
        Sv = (vm/vc)**(-1/2.) *(va/vm)**(-p/2.) *(v/va)**2.5 *Fvm
        Fnu_thick = Sv*(1.-tau)
        if (tauv <1.e-3):
            Fnu_thick = (vm/vc)**(-1/2.)*(v/vm)**(-p/2.)*Fvm
    if (v<=vc):
        Fnu=Fnu_thick *(v/vc)**(-1/2.)
    elif (v>vc) and (v<=va):
        Fnu=Fnu_thick
    elif (v>va):
        Fnu=Fnu_thick
    return Fnu

#Fv in slow cooling case        
def Fvslow(v,va,vm,vc,Fvm,p):
    if (va<=vm):
        tauv = (v/va)**(-5./3.)
        tau = np.exp(-tauv)
        Sv = (va/vm)**(1/3.)*(v/va)**2. *Fvm
        Fnu_thick = Sv*(1.-tau)
        if (tauv <1.e-3):
            Fnu_thick = (v/vm)**(1/3.)* Fvm
        if (v<=va):
            Fnu= Fnu_thick
        elif (v>va) and (v<=vm):
            Fnu= Fnu_thick
        elif (v>vm) and (v<=vc):
            Fnu= Fnu_thick* (v/vm)**(-(p-1.)/2.-1/3.)
        elif (v>vc):
            Fnu=Fnu_thick* (v/vm)**(-(p-1.)/2.-1/3.)*(v/vc)**(-1/2.)
    elif (va>vm) and (va<=vc):
        tauv = (v/va)**(-(p+4.)/2.)
        tau = np.exp(-tauv)
        Sv = (va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        Fnu_thick = Sv*(1.-tau)
        if (tauv <1.e-3):
            Fnu_thick = (v/vm)**(-(p-1.)/2.)*Fvm
        if (v<=vm):
            Fnu= Fnu_thick *(v/vm)**(-1/2.)
        elif (v>vm) and (v<=va):
            Fnu= Fnu_thick
        elif (v>va) and (v<=vc):
            Fnu= Fnu_thick
        elif (v>vc):
            Fnu= Fnu_thick* (v/vc)**(-1/2.)
    elif (va>vc):
        tauv = (v/va)**(-(p+5.)/2.)
        tau = np.exp(-tauv)
        Sv = (vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
        Fnu_thick = Sv*(1.-tau)
        if (tauv <1.e-3):
            Fnu_thick = (vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
        if (v<=vm):
            Fnu= Fnu_thick* (v/vm)**(-1/2.)
        elif (v>vm) and (v<=va):
            Fnu= Fnu_thick
        elif (v>va):
            Fnu= Fnu_thick
    return Fnu

#--------------------------end of update 1 for Fv------------------------------




#--------------------------Start update 2 for Fv------------------------------
#smooth for vm, vc:
#status: ongoing

def Fvslow2(v,va,vm,vc,Fvm,p):
    if (va<=vm):
        if (v<=va):
            Fnu=(va/vm)**(1/3.)*(v/va)**2. *Fvm
        elif (v>va):
            p1=p
            p2=p + 1.
            x1 = v/vm
            x2 = v/vc
            if (x2 < 10.): 
                if (x1 >20.):    x1 = 20.
    #                    s1= math.gamma(p1/4.+19/12.)*math.gamma(p1/4.-1/12.)
                f1 = lambda x: Fx(x) * x**((p1-3.)/2.)
                s1 = quad(f1,x2,x1)[0]
            else:
                s1 = 0.
                if (x2 > 20.): x2=20.
    #                s2= math.gamma(p2/4.+19/12.)*math.gamma(p2/4.-1/12.)
            f2 = lambda x: Fx(x) * x**((p2-3.)/2.)
            s2 = quad(f2,x2/100.,x2)[0]

                
            Fnu1 = (v/vm)**(-(p1-1.)/2.) * s1    
            Fnu2 =(vc/vm)**(-(p1-1.)/2.)*(v/vc)**(-(p2-1.)/2.)*s2

            Fnu = (Fnu1 + Fnu2) * Fvm
    elif (va>vm) and (va<=vc):
        if (v<=vm):
            Fnu=(vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vc):
        if (v<=vm):
            Fnu=(vc/va)**0.5* (vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    return Fnu

#the third version
#modified for smooth connection between transition        
def Fvslow3(v,va,vm,vc,Fvm,p):
    if (va<=vm):
        if (v<=va):
            Fnu=(va/vm)**(1/3.)*(v/va)**2. *Fvm
        elif (v>va) and (v<=vm):
            p1=p
            p2=p + 1.
            x1 = v/vm
            x2 = v/vc
            f1 = lambda x: Fx(x) * x**((p1-3.)/2.)
            s = quad(f1,x2,x1)[0]
            Fnu1 = (v/vm)**(-(p1-1.)/2.) * s

            f2 = lambda x: Fx(x) * x**((p2-3.)/2.)
            s = quad(f2,x2/10.,x2)[0]
            Fnu2 =(vc/vm)**(-(p1-1.)/2.)*(v/vc)**(-(p2-1.)/2.)*s

            Fnu = (Fnu1 + Fnu2) * Fvm
        elif (v>vm) and (v<=vc):
            p1=p
            p2=p + 1.
            x1 = 1.
            x2 = v/vc
            f1 = lambda x: Fx(x) * x**((p1-3.)/2.)
            s = quad(f1,x2,x1)[0]
            Fnu1 = (v/vm)**(-(p1-1.)/2.) * s

            f2 = lambda x: Fx(x) * x**((p2-3.)/2.)
            s = quad(f2,x2/10.,x2)[0]
            Fnu2 =(vc/vm)**(-(p1-1.)/2.)*(v/vc)**(-(p2-1.)/2.)*s

            Fnu = (Fnu1 + Fnu2) * Fvm
        elif (v>vc):
            p1=p
            p2=p + 1.
            f2 = lambda x: Fx(x) * x**((p2-3.)/2.)
            s = quad(f2,0.01,1.)[0]
            Fnu2 =(vc/vm)**(-(p1-1.)/2.)*(v/vc)**(-(p2-1.)/2.)*s

            Fnu = (Fnu2) * Fvm
    elif (va>vm) and (va<=vc):
        if (v<=vm):
            Fnu=(vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vc):
        if (v<=vm):
            Fnu=(vc/va)**0.5* (vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    return Fnu

def Get_N(a,b,width):
    N=int((b-a)/width + 1)
    if N%2 == 0:
        N=N+1
    return N


def Fvslow4(v,va,vm,vc,Fvm,p):
    if (va<=vm):
        if (v<=va):
            Fnu=(va/vm)**(1/3.)*(v/va)**2. *Fvm
        elif (v>va):
            p1=p
            p2=p + 1.
            x1 = v/vm
            x2 = v/vc
            f1 = lambda x: Fx(x) * x**((p1-3.)/2.)
            datas = []
            r=x2
            width = (x1-x2)/10.
            inum=Get_N(x2,x1,width)
            for i in range(0,inum):
                datas.append(f1(r))
                r = r+width
            sum = datas[0]+datas[inum-1]
            for i in range(2,inum):
                if i%2== 0:
                    sum = sum +4*datas[i-1]
                else:
                    sum = sum +2*datas[i-1]
            s=sum*width/3.0
            Fnu1 = (v/vm)**(-(p1-1.)/2.) * s

            f2 = lambda x: Fx(x) * x**((p2-3.)/2.)
            datas = []
            r=0.
            width = (x2-0.)/10.
            inum=Get_N(0.,x2,width)
            for i in range(0,inum):
                datas.append(f2(r))
                r = r+width
            sum = datas[0]+datas[inum-1]
            for i in range(2,inum):
                if i%2== 0:
                    sum = sum +4*datas[i-1]
                else:
                    sum = sum +2*datas[i-1]
            s=sum*width/3.0
            Fnu2 =(vc/vm)**(-(p1-1.)/2.)*(v/vc)**(-(p2-1.)/2.)*s

            Fnu = (Fnu1 + Fnu2) * Fvm
    elif (va>vm) and (va<=vc):
        if (v<=vm):
            Fnu=(vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vc):
        if (v<=vm):
            Fnu=(vc/va)**0.5* (vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    return Fnu

def Fvslow5(v,va,vm,vc,Fvm,p):
    if (va<=vm):
        if (v<=va):
            Fnu=(va/vm)**(1/3.)*(v/va)**2. *Fvm
        elif (v>va):
            p1=p
            p2=p + 1.
            x1 = v/vm
            x2 = v/vc
            if (x2 < 5.): 
                if (x1 >10.): 
                    s1= math.gamma(p1/4.+19/12.)*math.gamma(p1/4.-1/12.)
                else:
                    f1 = lambda x: Fx(x) * x**((p1-3.)/2.)
                    s1 = quad(f1,x2,x1)[0]
                if (x2 < 0.1): 
                    s2 = 0.
                else:
                    f2 = lambda x: Fx(x) * x**((p2-3.)/2.)
                    s2 = quad(f2,x2/100.,x2)[0]
            else:
                s1 = 0.
                s2= math.gamma(p2/4.+19/12.)*math.gamma(p2/4.-1/12.)
                
            Fnu1 = (v/vm)**(-(p1-1.)/2.) * s1    
            Fnu2 =(vc/vm)**(-(p1-1.)/2.)*(v/vc)**(-(p2-1.)/2.)*s2

            Fnu = (Fnu1 + Fnu2) * Fvm
    elif (va>vm) and (va<=vc):
        if (v<=vm):
            Fnu=(vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vc):
        if (v<=vm):
            Fnu=(vc/va)**0.5* (vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    return Fnu


def Fvslow6(v,va,vm,vc,Fvm,p):
    if (va<=vm):
        if (v<=va):
            Fnu=(va/vm)**(1/3.)*(v/va)**2. *Fvm
        elif (v>va):
            p1=p
            p2=p + 1.
            x1 = v/vm
            x2 = v/vc
            if (x1 <= 0.1):
                Fnu1 = (v/vm)**(1/3.)
                s2 =0.
            elif (x1 < 5.):
                f1 = lambda x: Fx(x) * x**((p1-3.)/2.)
                s1 = quad(f1,x2,x1)[0]
                Fnu1 = (v/vm)**(-(p1-1.)/2.)*s1
            else:
                Fnu1 = (v/vm)**(-(p1-1.)/2.)
                
            if (x2 > 5.):
                s2 =1.
            elif (x1 > 0.1):
                f2 = lambda x: Fx(x) * x**((p2-3.)/2.)
                s2 = quad(f2,x2/100.,x2)[0]

            Fnu2 =(vc/vm)**(-(p1-1.)/2.)*(v/vc)**(-(p2-1.)/2.)*s2

            Fnu = (Fnu1 + Fnu2) * Fvm
    elif (va>vm) and (va<=vc):
        if (v<=vm):
            Fnu=(vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(va/vm)**(-(p-1.)/2.) *(v/va)**2.5 *Fvm
        elif (v>va) and (v<=vc):
            Fnu=(v/vm)**(-(p-1.)/2.)*Fvm
        elif (v>vc):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    elif (va>vc):
        if (v<=vm):
            Fnu=(vc/va)**0.5* (vm/va)**((p+4.)/2.)*(v/vm)**2. * Fvm
        elif (v>vm) and (v<=va):
            Fnu=(vc/vm)**(-(p-1.)/2.) *(va/vc)**(-p/2.)*(v/va)**2.5 *Fvm
        elif (v>va):
            Fnu=(vc/vm)**(-(p-1.)/2.)*(v/vc)**(-p/2.)*Fvm
    return Fnu

def Fv1(v,va,vm,vc,Fvm,p):
    if (vc<=vm):
        Fnu=Fvfast1(v,va,vm,vc,Fvm,p)
    else:
        Fnu=Fvslow1(v,va,vm,vc,Fvm,p)
    return Fnu

def Fv(v,va,vm,vc,Fvm,p):
    if (vc<=vm):
        Fnu=Fvfast(v,va,vm,vc,Fvm,p)
#        Fnu=Fvfast_sbpl(v,va,vm,vc,Fvm,p)
    else:
        Fnu=Fvslow(v,va,vm,vc,Fvm,p)
#        Fnu=Fvslow_sbpl(v,va,vm,vc,Fvm,p)
    return Fnu



def vFv(v,va,vm,vc,Fvm,p):
    return v*Fv(v,va,vm,vc,Fvm,p)

#--------------------------------------------------------------------------
#---------------------------------SSC--------------------------------------
#--------------------------------------------------------------------------
#-----------------------Trail version of SSC
def FvIC_trail(v, gm_m, gm_c,vm, vc, Fvm_IC, p):
    if (vc<=vm):
#------fast cooling-------
        vm_IC=gm_c**2 *vm
        vc_IC=gm_m**2 *vc
        if (v<=vc_IC):
            Fnu=(v/vc_IC)**(1/3.)* Fvm_IC
        elif (v>vc_IC) and (v<=vm_IC):
            Fnu=(v/vc_IC)**(-1/2.)* Fvm_IC
        elif (v>vm_IC):
            Fnu=(vm_IC/vc_IC)**(-1/2.)*(v/vm_IC)**(-p/2.)*Fvm_IC        
    else:
#------slow cooling------
        vm_IC=gm_m**2 *vm
        vc_IC=gm_c**2 *vc
        if (v<=vm_IC):
            Fnu=(v/vm_IC)**(1/3.)* Fvm_IC
        elif (v>vm_IC) and (v<=vc_IC):
            Fnu=(v/vm_IC)**(-(p-1.)/2.)* Fvm_IC
        elif (v>vc_IC):
            Fnu=(vc_IC/vm_IC)**(-(p-1.)/2.)*(v/vc_IC)**(-p/2.)* Fvm_IC
    return Fnu

#-----------------------Simple broken power law (sbpl) version of SSC
#Fv in fast cooling case
def FvICfast_sbpl(v, gm_a,gm_m, gm_c,va,vm, vc,Fvm_IC, p):
    x0=0.4714
    vca=4.*gm_c**2 *va *x0
    vcm=4.*gm_c**2 *vm *x0
    vcc=4.*gm_c**2 *vc *x0
    vmm=4.*gm_m**2 *vm *x0
    vam=4.*gm_a**2 *vm *x0
    vaa=4.*gm_a**2 *va *x0
#    vmmax=4.*gm_m**2 *vm *x0
    if (va<=vc):
        if (v<=vca):
            Fnu=(vca/vcc)**(1/3.) *(v/vca)
        elif (v>vca) and (v<=vcc):
            Fnu=(v/vcc)**(1/3.)
        elif (v>vcc) and (v<=vmm):
            Fnu=(v/vcc)**(-1/2.)
        elif (v>vmm):
            Fnu=(vmm/vcc)**(-1/2.)*(v/vmm)**(-p/2.)
    elif (va>vc) and (va<=vm):
        if (v<=vaa):
            Fnu=(v/vaa)
        elif (v>vaa) and (v<=vmm):
            Fnu=(v/vaa)**(-1/2.)
        elif (v>vmm):
            Fnu=(vmm/vaa)**(-1/2.)*(v/vmm)**(-p/2.)    
    elif (va>vm):
        if (v<=vaa):
            Fnu=(v/vaa)
        else:
            Fnu=(v/vaa)**(-p/2.)
    return Fnu* Fvm_IC


#Fv in slow cooling case        
def FvICslow_sbpl(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p):
    x0=0.4714
    vca=4.*gm_c**2 *va *x0
    vma=4.*gm_m**2 *va *x0
    vmm=4.*gm_m**2 *vm *x0
    vmc=4.*gm_m**2 *vc *x0
    vaa=4.*gm_a**2 *va *x0
    vcc=4.*gm_c**2 *vc *x0
    if (va<=vm):
        if (v<=vma):
            Fnu=(vma/vmm)**(1/3.)*(v/vma) 
        elif (v>vma) and (v<=vmm):
            Fnu=(v/vmm)**(1/3.)
        elif (v>vmm) and (v<=vcc):
            Fnu=(v/vmm)**(-(p-1.)/2.)
        elif (v>vcc):
            Fnu=(vcc/vmm)**(-(p-1.)/2.)*(v/vcc)**(-p/2.)
    elif (va>vm) and (va<=vc):
        if (v<=vma):
            Fnu=(vmm/vma)**((p+1.)/2.)*(v/vmm)
        elif (v>vma) and (v<=vcc):
            Fnu=(v/vmm)**(-(p-1.)/2.)
        elif (v>vcc):
            Fnu=(vcc/vmm)**(1/2.)*(v/vmm)**(-p/2.)
    elif (va>vc):
        if (v<=vaa):
            Fnu=(v/vaa)
        else:
            Fnu=(v/vaa)**(-p/2.)
    return Fnu* Fvm_IC


#------------------------He & Lei's version of SSC------------
#Fv in fast cooling case
def FvICfast(v, gm_a,gm_m, gm_c,va,vm, vc, Fvm_IC, p):
    x0=0.4714
    vca=4.*gm_c**2 *va *x0
    vcm=4.*gm_c**2 *vm *x0
    vcc=4.*gm_c**2 *vc *x0
    vmm=4.*gm_m**2 *vm *x0
    vam=4.*gm_a**2 *vm *x0
    vaa=4.*gm_a**2 *va *x0
    if (va<=vc):
        if (v<=vca):
            Fnu=(va/vc)**(1/3.) *(v/vca) *Fvm_IC*5/6.
        elif (v>vca) and (v<=vcc):
            Fnu=(v/vcc)**(1/3.)* Fvm_IC*9/10.
        elif (v>vcc) and (v<=vcm):
            Fnu=(v/vcc)**(-1/2.)*Fvm_IC*(28/15.+np.log(v/vcc))/3.
        elif (v>vcm) and (v<=vmm):
            Fnu=(v/vcc)**(-1/2.)*Fvm_IC*(2*(p+5.)/(p+2.)/(p-1.)-2*(p-1.)/3./(p+2.)+np.log(vmm/v))/3.
        elif (v>vmm):
            Fnu=(vc/vm)*(v/vmm)**(-p/2.)*Fvm_IC*(2*(p+5.)/(p-1.)/3.-2*(p-1.)/(p+2.)/3.+np.log(v/vmm))/(p+2.)
    elif (va>vc) and (va<=vm):
        R=gm_c/gm_a/3.
        if (v<=vaa):
            Fnu=(0.5*R+1.)*(R+4.)*(v/vaa) *Fvm_IC
        elif (v>vaa) and (v<=vam):
            Fnu=R*(v/vaa)**(-1/2.)*Fvm_IC*(R/6.+0.9+R*np.log(v/vaa)/4.)
        elif (v>vam) and (v<=vmm):
            Fnu=R**2.*(v/vaa)**(-1/2.)*Fvm_IC*(3/(p-1.)-0.5+3.*np.log(vmm/v)/4.)
        elif (v>vmm):
            Fnu=R**2.*(va/vm)*(v/vmm)**(-p/2.)*Fvm_IC*(4/(p+3.)*(gm_a/gm_m)**(p-1.)*gm_a/gm_c+3*(p+1.)/(p-1./(p-2.+0.5*np.log(v/vmm))))*9./2./(p+2.)    
    elif (va>vm):
        R=(gm_m/gm_a)**(p-1.)*gm_c/gm_a/3.
        if (v<=vaa):
            Fnu=(3*R/2./(p+2.)+1.)*(3*R/(p+2.)+4.)*(v/vaa) *Fvm_IC
        elif (v>vaa):
            Fnu=(6*R/(p+3.)+R*(9*R/2/(p+2.)+1.)+9*R/4.*np.log(v/vaa))*(v/vaa)**(-p/2.)*Fvm_IC/(p+2.)
    return Fnu

#Fv in slow cooling case        
def FvICslow(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p):
    x0=0.4714
    vca=4.*gm_c**2 *va *x0
    vma=4.*gm_m**2 *va *x0
    vmm=4.*gm_m**2 *vm *x0
    vmc=4.*gm_m**2 *vc *x0
    vaa=4.*gm_a**2 *va *x0
    vcc=4.*gm_c**2 *vc *x0
    if (va<=vm):
        if (v<=vma):
            Fnu=(va/vm)**(1/3.)*(v/vma) *Fvm_IC*5*(p-1.)/(2*(p+1.))
        elif (v>vma) and (v<=vmm):
            Fnu=(v/vmm)**(1/3.)* Fvm_IC*3*(p-1.)/(2*(p-1/3.))
        elif (v>vmm) and (v<=vmc):
            Fnu=(v/vmm)**(-(p-1.)/2.)*Fvm_IC*(4*(p+1/3.)/(p+1.)/(p-1/3.)+np.log(v/vmm))*(p-1.)/(p+1.)
        elif (v>vmc) and (v<=vcc):
            Fnu=(v/vmm)**(-(p-1.)/2.)*Fvm_IC*(2*(2*p+3.)/(p+2.)-2/(p+1.)/(p+2.)+np.log(vcc/v))*(p-1.)/(p+1.)
        elif (v>vcc):
            Fnu=(v/vmm)**(-p/2.)*(vc/vm)*Fvm_IC*(2*(2*p+3.)/(p+2.)-2/(p+2.)**2.+np.log(v/vcc)*(p+1.)/(p+2.))*(p-1.)/(p+1.)
#        Fnu=(vcc/vmm)**(-(p-1.)/2.)*(v/vcc)**(-p/2.)*Fvm_IC
    elif (va>vm) and (va<=vc):
        if (v<=vma):
            Fnu=(vm/va)**((p+1.)/2.)*(v/vmm) * Fvm_IC*2*(p+4.)*(p-1.)/3./(p+1.)**2.
        elif (v>vma) and (v<=vmc):
            Fnu=(v/vmm)**(-(p-1.)/2.)*Fvm_IC*(2*(2*p+5.)/(p+1.)/(p+4.)+ np.log(v/vma))*(p-1.)/(p+1.)
        elif (v>vmc) and (v<=vca):
            Fnu=(v/vmm)**(-(p-1.)/2.)*Fvm_IC*(2.+2./(p+4.)+np.log(vc/va))*(p-1.)/(p+1.)
        elif (v>vca) and (v<=vcc):
            Fnu=(v/vmm)**(-(p-1.)/2.)*Fvm_IC*(2*(2*p+1.)/(p+1.)+np.log(vcc/v))*(p-1.)/(p+1.)
        elif (v>vcc):
            Fnu=(vc/vm)*(v/vmm)**(-p/2.)*Fvm_IC*(2*(2*p+5.)/(p+2.)+np.log(v/vcc))*(p-1.)/(p+1.)
    elif (va>vc):
        R=(p-1.)*(gm_m/gm_a)**(p-1.)*gm_c/gm_a/3.
        if (v<=vaa):
            Fnu=(3*R/2./(p+2.)+1.)*(3*R/(p+2.)+4.)*(v/vaa) *Fvm_IC
        elif (v>vaa):
            Fnu=(6*R/(p+3.)+R*(9*R/2/(p+2.)+1.)+9*R/4.*np.log(v/vaa))*(v/vaa)**(-p/2.)*Fvm_IC/(p+2.)
    return Fnu

#no KN, no vMax effects
def FvIC0(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p):
    if (vc<=vm):
#        Fnu=FvICfast(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
        Fnu=FvICfast_sbpl(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
        
    else:
#        Fnu=FvICslow(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
        Fnu=FvICslow_sbpl(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
    return Fnu


def FvIC(v, gm_a,gm_m, gm_c, va, vm, vc, vMax, Fvm_IC, GM, p):
    if (vc<=vm):
#        Fnu=FvICfast(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
        Fnu=FvICfast(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
        x=cgs.h*v/(GM*cgs.me*cgs.c**2. *gm_m)
    else:
#        Fnu=FvICslow(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
        Fnu=FvICslow(v, gm_a,gm_m, gm_c, va, vm, vc, Fvm_IC, p)
        x=cgs.h*v/(GM*cgs.me*cgs.c**2. *gm_c)
    sigmaR=astro.sigmaKN(x)/cgs.sigmaT
    return Fnu*sigmaR



#==================================Physical Constants useful in Astronomy, in cgs units.=================================
#Credit: Weihua Lei
#Institue for Particle Physics and Astrophysics, Huazhong University of Science and Technology (HUST), Wuhan, China
#Email: leiwh@hust.edu.cn
#Homepage: http://www.physics.unlv.edu/~leiwh/pygrb/
#
#  Description:
#Created on Tue Aug 16 19:39:07 2011, University of Nevada Las Vegas (UNLV), USA
#Current version: 2014-11-25
#
#
#==============================================================================

#!/usr/bin/python
# Filename: cgs.py
from astropy import constants as const
import numpy as np

def about():
	print("List of the constants in cgs")
version = '2.0'

def help():
    print("list")

zero=np.exp(-700)
day=24.0*3600.0
year=day*365.0
deg=180.0/3.14159
pi=3.141592653589793

#Speed of light
#c=2.99792458*1.e10
c=const.c.cgs.value
#Plank's constant
#h= 6.6260755e-27
h=const.h.cgs.value

#Reduced Planck constant
#hbar=h/2pi
#hbar=1.05457266e-27
hbar=const.hbar.cgs.value

#Boltzmann's constant
#k=1.380658e-16
k_B=const.k_B.cgs.value
k=k_B

#Electron charge
#eq=4.8e-10
e=const.e.gauss.value
eq=e
#qe=4.8e-10
qe=eq

#---mass
#electron mass
#me=9.11e-28
m_e=const.m_e.cgs.value
me=m_e
#proton mass
#mp=1.6726231e-24
mp=const.m_p.cgs.value
#neutron mass
#mn=1.674929e-24
mn=const.m_n.cgs.value
mH=1.673534e-24

#Gravitational constant
#G=6.67259e-8
G=const.G.cgs.value

#Avogadro number
#NA=6.022e23
N_A=const.N_A.cgs.value
NA=N_A

#Atomic mass
u=1.66e-24
#u=const.u.cgs.value

#Fine-structure constant
aa=7.297e-3
alpha= aa
#alpha=const.alpha.cgs.value
aa=alpha

qm=5.273e17
R0=1.097e5
a0=5.29e-9
lamda=2.43e-10
re=2.82e-13

#Gas constant
#R=8.31e7
R=const.R.cgs.value

#Stefan-Boltzmann constant
#sigma=5.67e-5
sigma_sb=const.sigma_sb.cgs.value
sigma=sigma_sb

#Thomson cross section
sigmaT=6.65e-25
#Radiation constant: a=4sigma/c
a=7.566e-15


#---distance
#Astronomical Unit 
#AU=1.496e13
au=const.au.cgs.value
AU=au
#A=1.496e13
A=au

#Kiloparsec
#pc=3.086e18
pc=const.pc.cgs.value
#kpc=3.086e21
kpc=const.kpc.cgs.value
Mpc=3.085677581467192e24

#pc2A=206264.806
#pc2AU=206264.806
pc2AU=const.pc.to('au').value
pc2A=pc2AU

pc2ly=3.26
ly=9.461e17
ly2A=6.324e4
ly2pc=0.3066
mile=160934.4

#--energy, frequence, flux
eV2cm=12398.52e-8
eV2bs=8065.479
eV2Hz=2.418e14
keV2Hz=2.418e17
MeV2Hz=2.418e20
eV2erg=1.6e-12
keV2erg=1.6e-9
MeV2erg=1.6e-6
eV2K=11604.5
keV2K=eV2K*1e3
MeV2K=eV2K*1e6
K2eV=1./eV2K
K2keV=1./keV2K
K2MeV=1./MeV2K
erg=1.e-7
esu=0.33e-9
dyn=1.e-5
mecc2MeV=0.511
Jy=1.e-23
mJy=1.e-26
uJy=1.e-29
Crab=2.4e-8
mCrab=2.4e-11


#---Sun, Earth, Jupiter
#Solar mass
#Msun=1.9891e33
M_sun=const.M_sun.cgs.value
Msun=M_sun

#Solar radius
#Rsun=6.9599e10
R_sun=const.R_sun.cgs.value
Rsun=R_sun

#Solar distance to earth
Dsun2earth=1.5e13

#Solar luminosity
#Lsun=3.826e33
L_sun=const.L_sun.cgs.value
Lsun=L_sun

#distance from Sun to Sgr A*
Dsun=8.33*kpc

#Earth mass
#Mearth=6.0e27
M_earth=const.M_earth.cgs.value
Mearth=M_earth

#Earth radius
#Rearth=6.4e8
R_earth=const.R_earth.cgs.value
Rearth=R_earth

#Jupiter mass
#MJupiter=1.9e30
M_jup=const.M_jup.cgs.value
MJupiter=M_jup

#Jupiter radius
#RJupiter=7.1e9
R_jup=const.R_jup.cgs.value
RJupiter=R_jup

#Moon mass
Mmoon=7.4e25
#Moon radius
Rmoon=1.7e8
#Moon distance to earth
Dmoon=3.8e10

#-------other
#inch to cm
inch=2.539999918


#---Other unit in astrophysics
Msun2erg=Msun* c**2.
rg=G* Msun/c**2.
UnitOmega=c**3/G/Msun


# End of units.py



import numpy as np
import matplotlib.pyplot as plt
from PyFRS import cgs,fs
if __name__ == '__main__':
    Z={ 'z':         0.1,              # Redshift
        'k':         0,                # Medium density profile index
        'SSC':       'No',             # SSC
        'XIC':       'No',             # EIC
        'E0':        1.e53,            # Isotropic-equivalent energy in erg
        'Gamma0':    2.,               # Initial bulk Lorentz factor
        'theta_j':   5.,               # Jet opening angle
        'theta_obs': 2.8,              # Viewing angle in deg
        'n18':       1.,               # CNM density at 10^18cm
        'p':         2.2,              # Electron energy distribution index
        'epsilon_e': 0.1,              # Fraction of the shock energy into electrons
        'epsilon_B': 0.01 }

    ta = 1.0e-1 * cgs.day
    tb = 1.0e3 * cgs.day
    t = np.geomspace(ta, tb, num=100)

    nuR=np.empty(t.shape)
    nuO=np.empty(t.shape)
    nuX=np.empty(t.shape)

    nuR[:] = 6.0e9
    nuO[:] = 1.0e14
    nuX[:] = 1.0e18

    print("Calc Radio")
    FnuR = fs.FS_flux(t, nuR, **Z)
    print("Calc Optical")
    FnuO = fs.FS_flux(t, nuO, **Z)
    print("Calc X-ray")
    FnuX = fs.FS_flux(t, nuX, **Z)
    print("Plot")

    tday = t / cgs.day

    fig, ax = plt.subplots(1, 1)
    ax.plot(tday, FnuR, ls='-', label=r'$\nu=6$ GHz')
    ax.plot(tday, FnuO, ls='--', label=r'$\nu=10^{14}$ Hz')
    ax.plot(tday, FnuX, ls='-.', label=r'$\nu=10^{18}$ Hz')
    ax.set_xscale('log')
    ax.set_yscale('log')
    ax.set_xlabel(r'$t$ (d)')
    ax.set_ylabel(r'$F_\nu$ (mJy)')
    ax.legend()
    fig.tight_layout()

    print("Saving lc_multi.png")
    fig.savefig("lc_multi.png")
    plt.close(fig)


#==================================Astrophysics Fundaments=================================
#Credit: Weihua Lei
#Institue for Particle Physics and Astrophysics, Huazhong University of Science and Technology (HUST), Wuhan, China
#Email: leiwh@hust.edu.cn
#Homepage: http://www.physics.unlv.edu/~leiwh/pygrb/
#
#  Description:
#Created on Tue Aug 16 19:39:07 2011, University of Nevada Las Vegas (UNLV), USA
#Current version: 2014-11-25
#
#
#==============================================================================

import math
from PyFRS import cgs
import numpy as np
from scipy.integrate import quad
#from cosmocalc import cosmocalc
from PyFRS import cosmocalc


# Hubble constant, cm/s/cm
H_0=72.0 *1.0e5/(1.0e6 *cgs.pc); 
def flumdist(x):
   return 1.0/math.sqrt(0.3 *(1.0+x)**3.0 + 0.7)

def lumdist(z=None):
    if (z is None):
        ss="Syntax: lumdist(z)"
    else:    
        finte=quad(flumdist,0,z)[0]
        ss=cgs.c*(1.0+z)/H_0 *finte
    return ss

def LUD(z):
    return cosmocalc.cosmocalc(z)['DL_cm']


#Klein-Nishina scattering cross section
def sigmaKN(x):
   if (x > 1.e-4):
      s1=(1.+x)/x**3. *(2.*x*(1.+x)/(1.+2.*x)-np.log(1.+2.*x) )
      s2=1./(2.*x) *np.log(1+2.*x) - (1.+3.*x)/(1.+2.*x)**2.
      s=0.75*cgs.sigmaT*(s1+s2)
   else:
      s=cgs.sigmaT
   return s


def band(UBV,key):
    if (key is None):
        ss= "Syntax: band(UBV,key); About: Standard apparent magnitudes and fluxes; flux in Jy, lambda in um; nu in Hz"
    else:
        flux={
            'U': [0.36, 1810],
            'B': [0.22, 4260],
            'V': [0.55, 3640],
            'R': [0.64, 3080],
            'I': [0.79, 2550],
            'J': [1.26, 1600],
            'H': [1.6, 1080],
            'K': [2.22, 670],
            'g': [0.52, 3730],
            'r': [0.67, 4490],
            'i': [0.79, 4760],
            'z': [0.91, 4810]
        }
        ikey = {'lambda': 0, 'flux': 1}
        if (key == 'nu'):
            ss= 1.e4*cgs.c/flux[UBV][ikey['lambda']]
        else:
            ss = flux[UBV][ikey[key]]
    return ss

def mag(UBV, m=None):
    if (m is None):
        ss= "Syntax: mag(UBV,m); About: aparent magnitudescy, unit: erg cm^-2 Hz^-1 s^-1"
    else:
        ss=band(UBV,'flux')*cgs.Jy*100.0**(-m/5.)
    return ss


def m2MAG(d, m=None):
    if (m is None):
        ss= "Syntax: m2MAG(d, m); apparent mag (m) to absoulte mag (M); About: distance d in pc"
    else:
        ss= m - 5.* np.log10(d/10.)
    return ss


def Msmbh(Lbul=None):
    if (Lbul is None):
        ss= "Syntax: MBH(Lbulge); About: BH mass of galatic center SMBH with M-Lbul relation, unit: Msun; for Lbul: Lsun"
    else:
        ss=10.**(8.07+1.26*(np.log10(Lbul)-10.) )
    return ss

def darcsec(z=None):
    if (z is None):
        ss= "Syntax: darcsec(z); About: distance for 1 arcsec for object with known redshift, where z=0 for objects in Milky Way, unit: kpc"
    else:
        if z==0.0:
            dl = cgs.Dsun
        else:
            dl= LUD(z)
            da=dl/(1.+z)**2.
        ss= da*cgs.pi/180./3600./cgs.kpc 
    return ss

    
def nu_cy(B=None):
    if (B is None):
        ss= "Syntax: nu_cy(B); About: cyclotron frequency"
    else:
        ss=cgs.eq*B/(2.*cgs.pi*cgs.me*cgs.c)
    return ss
    
def r_A(B=None, Mns=1.4, rns=1e6,mdot=5e-17):
    if (B is None):
        ss= "Syntax: r_A(B,Mns, rns,mot); About: Alfven radius"
    else:
        ss=(B**4.*rns**12./(2.*cgs.G*Mns*cgs.Msun*3.*mdot**2.))**(1/7.)
    return ss

def t_ff(rho=None):
    if (rho is None):
        ss= "Syntax: t_ff(rho); About: free-fall time for a cloud that has satisfied the Jeans criterion"
    else:
        ss=(3.*cgs.pi/(32.*cgs.G*rho))**0.5
    return ss
    
def R_J(rho=None, T=50.):
    mu=1.
    if (rho is None):
        ss= "Syntax: R_J(rho, T); About: Jeans length"
    else:
        ss=(15.*cgs.k *T/(4.*cgs.pi*cgs.G*mu*cgs.mH*rho))**0.5
    return ss    


def gamma(beta=None):
    if (beta is None):
        ss= "Syntax: gamma(beta); About: velocity to Gamma"
    else:
        ss=(1/(1-beta**2))**0.5
    return ss    

def beta(gamma=None):
    if (beta is None):
        ss= "Syntax: beta(gamma); About: gamma to velocity"
    else:
        ss=(1-1/gamma**2)**0.5
    return ss    



from distutils.core import setup
setup(name='PyFRS',# 包名
      version='0.0.9.1',
      description='Compute the multiband emissions from Forward-Reverse Shocks of a GRB or TDE jet',
      author='Wei-Hua Lei',
      author_email='leiwh78@gmail.com',
      packages=['PyFRS'],
      )



# -*- coding: utf-8 -*-
"""
Created on Thu Dec  7 21:28:54 2017

@author: leiwh
"""
#-----------Scientific Library-------
import math
import numpy as np
import scipy.optimize

#-----------My Library---------
#Credit: Weihua Lei
from PyFRS import cgs,astro,grb

# try:
#     from . import cgs,astro,grb
# except ImportError:
#     import cgs,astro,grb




def FS_flux(time_obs,nu_obs,**Z):
    log_obs_time = np.log10(time_obs)

#-------Unpack parameters
    for key,value in Z.items():
        if key == 'z':              # Redshift
            z = value
        elif key == 'k':            # Medium density profile index
            k = value
        elif key == 'SSC':          # SSC
            # if value == 1:
            #     SSC = 'Yes'
            # else:
            #     SSC = 'No'
            SSC = value
        elif key == 'XIC':          # EIC
            # if value == 1:
            #     XIC = 'Yes'
            # else:
            #     XIC = 'No'
            XIC = value
        elif key == 'E0':           # Isotropic-equivalent energy in 10^52 erg
            E52 = value/1.0e52
        elif key == 'Gamma0':       # Initial bulk Lorentz factor
            Gm0 = value
        elif key == 'theta_j':      # Jet opening angle
            theta_j = value
        elif key == 'theta_obs':    # Viewing angle in deg
            theta_obs = value
        elif key == 'n18':          # CNM density at 10^18cm
            n18 = value
        elif key == 'p':            # Electron energy distribution index
            p = value
        elif key == 'epsilon_e':    # Fraction of the shock energy into electrons
            epsilon_e = value
        elif key == 'epsilon_B':    # Fraction of the shock energy in magnetic field
            epsilon_B = value



#==============Forward shock=====================
#-------Input parameters: independent of model
    zi=z
    D28=astro.LUD(zi)/1.e28

#        k=0.
    
    n_dj=0.
    t_djs=0.
    t_dje=0.
    
    Lj50=0.
    q=0.1
    t_ej=1000.
    t_ejp=2000.
    t_ejf=5000.
    
    mytime=1.*cgs.day
    


    # Band={'X-ray': 1e18,'Optical': 1e15,'Radio': 1e9, 'GeV': 2.4e23}
#        Time={'input':tday, 'adjust':10.0**lgtday}
    Dynmodel={'analytical': 1,'differential': 2}
    with_SSC={'Yes':1, 'No':0}
    with_jetbreak={'Yes':1, 'No':0}
    Tunits = {'Second': 1.0, 'Day': cgs.day}
    Nuunits= {'Hz': 1.0, 'keV': cgs.keV2Hz}

    dynamic_model='differential'
    radiative_efficiency = 'No'

    # Band_mode='X-ray'
#        v=Band[Band_mode]
    vlist=list(dict.fromkeys(list(nu_obs)))
    Funit=1.
    time_unit='Second'
    nu_unit='Hz'
    jet_break='Yes'
    inj_model='No'
    Smooth='No'
    # SSC = 'Yes'
    # XIC = 'Yes'


# #-------refresh output parameters for jet
#     t_dec=1.0
#     t_Sed=1.0
#     t_jet=1.0
#     t_col=1.0
#     t_min=1.0
#     t_colmin=1.0
#     nu_a=1.0
#     nu_min=1.0
#     nu_cool=1.0

#-------Input for jet

    n1=n18
    pp=p
    inum=500
    # inumv=500
    # inump=20

#-------Initialize the time, gamma, Fv arries in Log space
    t=np.logspace(-4., 3.0,inum)
    to=np.logspace(-4., 3.0,inum)  # t_observe

    Rt=np.logspace(14.0,21.0,inum)        # original value: 14. to 21.
    Gmt=np.logspace(2.,-5.,inum)

#        Coolingt=np.zeros((inum,6))

    para=np.zeros((inum,19))
    lgt=np.logspace(-4., 3.0,inum)
    lgFv=np.logspace(1.,6.,inum)*np.ones((len(vlist),inum))
    lgFv_SSC=np.logspace(1.,6.,inum)*np.ones((len(vlist),inum))

    # lgNu=np.logspace(7.,30.,inumv)
    # lgFnu=np.logspace(1.,6.,inumv)
    # lgFnu_SSC=np.logspace(1.,6.,inumv)

#-------Initialize critical parameters
    # lgNua=np.logspace(-4.,3.0,inump)
    # lgNum=np.logspace(-4.,3.0,inump)
    # lgNuc=np.logspace(-4.,3.0,inump)
    # lgFluxa=np.logspace(-4.,3.0,inump)
    # lgFluxm=np.logspace(-4.,3.0,inump)
    # lgFluxc=np.logspace(-4.,3.0,inump)

    # lgtdec=np.logspace(-4.,3.0,inump)
    # lgFluxdec=np.logspace(-4.,3.0,inump)
    # lgtSed=np.logspace(-4.,3.0,inump)
    # lgFluxSed=np.logspace(-4.,3.0,inump)

#-------start calculation
    # Gm0=self.Gamma0
    Gmt[1]=Gm0

    R0=Rt[1]
    Eiso=1.e52 *E52
    t[0]=0.
    t[1]=R0/(2.*Gm0**2.0*cgs.c*cgs.day)
    to[0]=0.
    to[1]=t[1]

#        if (lgtday<math.log(t[1],10)):
#            lgtday=math.log(t[1],10)


#        Rdec=(3.*Eiso/(Gm0**2* cgs.c**2 *4.*cgs.pi*n*cgs.mp))**(1/3.)


#-------jet, LoS, view angle
    # self.theta_obs=theta_obs_j
    thetaj=theta_j/cgs.deg
    thetaobs=theta_obs/cgs.deg
    fb=(1.-np.cos(thetaj))*2.*cgs.pi/(4.*cgs.pi)
    # if (theta_obs <= theta_j):
    #     theta_view=0.
    # else:
    #     theta_view=(thetaobs - thetaj)
    m0=1.e-5
    dm=0.0
#-------true jet energy: E_j=E_iso*f_b
    Ej=Eiso*fb
    Mej0=Ej/((Gm0-1.)*cgs.c**2.0)
    epsilon_rad=0.0
    beta=1.0

    vm0=0.0
    vc0=0.0
    fview=1.
    fj=1.
    dE=0.0
    dva=0.0
    dvai=0.0
#        m0=2.*cgs.pi*R0**3.*(1.-math.cos(theta_j))*n*cgs.mp/3.
    for i in range(1,inum):
        ni=n18*(Rt[i]/1.e18)**(-k)
        n1=ni
        if (i==1):
            Gmt[i]=Gm0
            beta=np.sqrt(1.-1/Gmt[i]**2.0)
            #fview=(1.-beta)/(1.-beta*np.cos(theta_view) )
            t[i]=R0/(2.*Gm0**2.0*cgs.c) *(1.+zi)
            mt=m0
        else:
            dR=Rt[i]-Rt[i-1]
            dm=2.*cgs.pi*Rt[i]**2.*(1.-np.cos(thetaj))*n1*cgs.mp*dR
            mt=mt+dm
            if (Dynmodel[dynamic_model]==1):
#-------------------if (dynamic_model=="analytical")
                Gmt[i] = grb.Gammat(Gm0,n1, Eiso, Rt[i],R0)
            else:
#-------------------if (dynamic_model=="differential")
#                    dm=2.*cgs.pi*Rt[i]**2.*(1.-math.cos(theta_j))*n*cgs.mp*dR
                tj=t[i-1]
                if (inj_model == 'No'):
                    dE=0.
                
                elif ((tj > t_ej) and (tj< t_ejf)):
                    ts=t_ej
                    if (inj_model == 'fallback'):
                        tp=t_ejp
                        s=0.5
                        Lj=fb*Lj50*1.e50 *(0.5*((tj-ts)/(tp-ts))**(-0.5*s) + 0.5*((tj-ts)/(tp-ts))**(q*s) )**(-1./s)
                    else:
                        Lj=fb*Lj50*1.e50* (tj/ts)**(-q)

                    dE=(1.-beta)/beta/cgs.c/cgs.c/cgs.c * Lj*dR
                dGm=-((Gmt[i-1]**2.-1.)*dm - dE)/(Mej0+mt*epsilon_rad+2.*(1.-epsilon_rad)*Gmt[i-1]*mt)
                Gmt[i]=Gmt[i-1]+dGm
            beta=np.sqrt(1.-1/Gmt[i]**2.0)
            dtb=dR/(beta*cgs.c)
            dt=dtb/Gmt[i]/(Gmt[i]+np.sqrt(Gmt[i]**2.0-1.0)) *(1.+zi)
#---------------effect of on-axis and off-axis
            #fview=(1.-beta)/(1.-beta*math.cos(theta_view) )
            t[i]=t[i-1]+dt  #central engine time
        GM21=Gmt[i]

#---------------effect of on-axis and off-axis
        theta_view=grb.theta_off(GM21,thetaobs,thetaj)
        fview=grb.aoff(GM21,theta_view)

        ti=t[i]/fview  #off-viewer's time
        to[i]=ti
        Ri=Rt[i]
        lgt[i]=np.log10(ti/Tunits[time_unit])

#-----------add density-jump
        if (n_dj>0.) and (ti>=t_djs) and (ti<=t_dje):
            n1=n_dj
        else:
            n1=ni


# #-----------t_dec and t_Sed-------------------
# #            if ((mt-dm)*(Gmt[i-1]-1) <= Mej0) and (mt*(Gmt[i]-1) > Mej0):
#         if ((mt-dm)*Gm0 <= Mej0) and (mt*Gm0 > Mej0):
# #                print("### Deceleration:", Rt[i-1], grb.Rdec(Gm0,n1,Eiso), ti/Tunits[time_unit])
#             t_dec=ti/Tunits[time_unit]

#         if ((mt-dm) *cgs.c**2 <= Eiso*fb) and (mt*cgs.c**2 > Eiso*fb):
#             Rsed0=grb.Rsed(n1,Eiso*fb)
# #                if (Rt[i-1] <= Rsed0) and (Rt[i] > Rsed0):
# #                print("### Sedov:", Rt[i-1], Rsed0, ti/Tunits[time_unit], Gmt[i])
#             t_Sed=ti/Tunits[time_unit]


#-----------Internal energy---------
        ei=grb.e2(GM21,n1)
        n2=grb.n2_c(GM21,n1)

#-----------B field at comoving frame-------
        Bc=grb.Bco2(epsilon_B,ei)

#-----------Max Lorentz factor-------
        gm_Max = grb.gamma_Max(Bc)
#-----------Minimum Lorentz factor-------
        gm_m=grb.gamma_m2(epsilon_e,GM21,pp,gm_Max)
        gm_m0=gm_m
#-----------Cooling Lorentez factor
        gm_c=grb.gamma_c(Bc,GM21,ti*fview,zi)
        gm_c0=gm_c

#----------For deep Newtonian Phase----
#Introduce a factor fDN due to deep Newtonian
#See Sironi & Giannios, 2013, ApJ, 778, 107
        gm_syn = 1.
        fDN = 1.
        if (gm_m0 <= gm_c0):
            if (gm_m0 < gm_syn):
                gm_m=gm_syn
                fDN = gm_m0/gm_syn
                if (pp > 3.):
                    fDN = (gm_m0/gm_syn)**((-1.+pp)/2.)
        else:
            if (gm_c0 < gm_syn):
                gm_c0=gm_syn
                fDN=gm_c0
#                    fDN_n = ((gm_syn_n -1.)/(gm_m_c -1.))**(1.-2.)
        

#-----------Y-parameter for SSC without K-N correction
        UB = Bc**2. /(8.*cgs.pi)
#            Coolingt[i,0]=ti/Tunits[self.time_unit]
#            Coolingt[i,2]=UB

        Y_SSC = 0.
        Uph=1.
        
        if SSC == 'Yes':
            if gm_c <= gm_m:
                eta_e = 1.
            else:
                eta_e = (gm_c/gm_m)**(2.-pp)
            ratio_epsilon = epsilon_e/epsilon_B
            Y_SSC = (-1. + math.sqrt(1.+4.* eta_e* ratio_epsilon))/2.
#                Coolingt[i,4]=UB* Y_SSC
            Uph=UB* Y_SSC
#                Coolingt[i,5]=Y_SSC

#-----------IC cooling of electrons by X-ray photons from internal shock, see Kumar et al. 2013
        if (XIC == 'Yes'):
#---------------X-ray flux from interal shock
            #Lx = 5.e46  # X-ray flux for SwJ1644+57
            #Lx = 10**48.2318 * t_n[i]**(-1.4)
            #Lx = 10**48.5 * (t_n[i]/cgs.day)**(-1.4)
            td_ce=t[i]/cgs.day #central engine time in unit of day
            if td_ce < 20.:
                #if t_n[i]/cgs.day < 5.:
                    #Lx = 50.* 4. *10**46
                #else:
                Lx = 4. *10**46
                #print(t_n[i]/cgs.day, ', R= ',Ri_n, ', B=', Bc_n, 'n=', n1_n)
            else:
                Lx = 4. *10**46 * (td_ce /20.)**(-5/3.)
#---------------Comoving frame magnetic energy density and IC photons from interal shock
            Ux = Lx/(16.*cgs.pi* GM21**2. * Ri**2. *cgs.c)
#                UB_n = Bc_n**2. /(8.*cgs.pi)
            Y_XIC = Ux/UB
#                Coolingt_n[i,0]=t_n[i]/Tunits[self.time_unit]
#                Coolingt[i,1]=Ux
#                Coolingt_n[i,2]=UB_n
#                Coolingt[i,3]=Y_XIC
            #gm_c_n=grb.gamma_c(Bc_n,GM21_n,ti_n,zi) /(1. + Y_n)

# #---------------Include the coorections due to the K-N cross-section
#             betax = 0.7
#             nux = 10.* 1.e-3 *(1.+zi)/GM21
#             rex = (cgs.mecc2MeV/nux)**(1.-betax)
#             Cx = Y_XIC* rex

#             def fkn(ge): 
#                 return Cx* ge**betax + ge* (1.+ Y_SSC) -gm_c

#             if Cx >= (gm_c -1.):
#                 gm_c_kn =1.
#             else:
#             #gm_c_n_kn = scipy.optimize.newton(fkn,1.)
#                 gm_c_kn = scipy.optimize.brenth(fkn,1., gm_c)
#             #gm_c_n_kn = gm_c_n/(1.+ Y_n* rex)
#             #print(gm_c_n_kn)
#             gm_c = gm_c_kn
#         else:
#             gm_c = gm_c/(1.+ Y_SSC)

#-----------v_Max, v_m, v_c, v_a----------
        vMax = grb.nu_gme(GM21,gm_Max, Bc,zi)
        vm=grb.nu_gme(GM21,gm_m, Bc,zi)
        vc=grb.nu_gme(GM21,gm_c, Bc,zi)
        va=grb.nu_aB(Bc,pp,n1,vm,vc,GM21,Ri,zi)

#      inlcude the radiative efficiency in dyanmics : 2025-5-24
        if radiative_efficiency == 'Yes':
            if (gm_m > gm_c):
                epsilon_rad=1.
            else:
                if (pp >= 3.):
                    epsilon_rad=gm_m/gm_c
                elif (2. < pp < 3.):
                    epsilon_rad=(gm_c/gm_m)**(2.-pp)
                else:
                    epsilon_rad= 1.    
                #epsilon_rad=(gm_c/gm_m)**(2.-pp)
            epsilon_rad=epsilon_rad * epsilon_e

#            print('t=',t_n[i]/cgs.day, ', vMax=',vMax_n,', vm=', vm_n, ', vc=',vc_n)

#            vat[i,0]=np.log10(ti/Tunits[self.time_unit])
#            vat[i,1]=np.log10(va*fview)
#            vat[i,2]=np.log10(vm*fview)
#            vat[i,3]=np.log10(vc*fview)
#            vat[i,4]=np.log10(vMax*fview)

# #-----------t_min, t_col and t_colmin, t_jet
        if (i>1):
#             if ((v - vm) * (v - vm0) <= 0.0 ):
# #                    print("### time for vm cross:", Rt[i-1], ti/Tunits[self.time_unit])
#                 t_min=ti/Tunits[time_unit]
#             if ((v - vc) * (v - vc0) <= 0.0 ):
# #                    print("### time for vc cross:", Rt[i-1], ti/Tunits[self.time_unit])
#                 t_col=ti/Tunits[time_unit]
            if ((vc - vm) * (vc0 - vm0) <= 0.0 ):
# #                    print("### time for vc=vm:", Rt[i-1], ti/Tunits[self.time_unit])
#                 t_colmin=ti/Tunits[time_unit]

#-----------dva is introduced for smooth connectiong of va during vm vc transition    
#                c0= 2.8071453296870878e-09
#                cnugm=3.575013703788261e-07
#                Cg1=4.*GM21*n1*(pp-1.)*gm_m**(pp-1.)
#                va1= ((c0/Bc)*(pp+2.)/(pp+2./3.)*Cg1*(cnugm/Bc)**(-5./3.)*gm_m**(-(pp+2./3.)) *Ri/GM21)**(3./5.) *GM21/(1.+zi)
#                Cg2=4.*GM21*n1*gm_c
#                va2= ((c0/Bc)*(2+2.)/(2+2./3.)*Cg2*(cnugm/Bc)**(-5./3.)*gm_c**(-(2+2./3.)) *Ri/GM21)**(3./5.) *GM21/(1.+zi)
#                dva=va1/va2
                va1=(pp+2.)/(pp+2./3.)*(pp-1.)
                va2=(2+2.)/(2+2./3.)
                dva=(va1/va2)**(3/5.)
#                print("### the ratio of va when vc=vm:",dva)

        if (dva != 0.0):
#                dvai=dva*(ti/self.t_colmin)**(-1.)
            dvai=dva
#                print(dvai)
            if (vc< vm):
                va=va*dvai
            else:
                va=va/dvai

        gm_a=(va*(2.*cgs.pi *cgs.me *cgs.c)*(1.+zi)/(GM21 *cgs.qe *Bc))**0.5




#-----------replace vm0 and vc0 for next loop
        vm0=vm
        vc0=vc

#-----------Jet break time: t_jet
        if (with_jetbreak[jet_break]==1):
            if (thetaj < (1./GM21) ):
                if (fj == 1.):
#                        print("### jet break time:", ti/Tunits[self.time_unit])
                    t_jet=ti/Tunits[time_unit]
                fj=thetaj**2. /(1./GM21**2.)



#-----------Corrections on off-axis jet for flux, see Salafia et al. 2016, arXiv:1601.03735
        # if (thetaj<1./GM21):
        #     thetaj = 1./GM21

        # thetajs=thetaj-1/GM21
        # if (thetaobs <= thetajs):
        #     fviewF=1.
        # elif ((thetaobs > thetajs) and (thetaobs <= thetaj)):
        #     fviewF = 1. - GM21*(thetaobs-thetajs)/2.
        # else:
        #     fviewF=0.5* fview**(3.-np.sqrt(2.)*thetaj**(1/3.))


#-----------Pvm and Fvm
        Pvm=grb.Pvmax(GM21,Bc)


        for l,v in enumerate(vlist):
#-----------Corrections on flux due to jet break effect
            Ne=mt/cgs.mp*2./(1.-math.cos(thetaj))
            Fvm=(1+zi)*Ne*Pvm/(4.*cgs.pi*(D28*1e28)**2.)*fj*fDN/cgs.uJy

            Fvm_IC=1.

#-----------Corrections on off-axis jet for flux, see Beniamini et al. 2023
            fviewF=grb.FvOff(GM21,thetaobs,thetaj)


    #            Fvti=fview**3. *grb.Fv(v/fview,va,vm,vc,Fvm,pp)  *Funit
            if (Smooth=='Yes'):
                Fvti=fviewF *grb.Fv(v/fview,va,vm,vc,Fvm,pp)  *Funit
            else:
                Fvti=fviewF *grb.Fv_sbpl(v/fview,va,vm,vc,Fvm,pp)  *Funit
                
            if (v/fview>=vMax):
                if (v/fview/vMax > 300.):
                    Fvti=Fvti*np.exp(-300.+1.)
                else:
                    Fvti=Fvti*np.exp(-v/fview/vMax+1.)
    #
            lgFv[l][i]=np.log10(Fvti)
    #            Fvt[i,0]=lgt[i]
    #            Fvt[i,1]=lgFv[i]
    #            LC[i,0]=lgt[i]
    #            LC[i,1]=lgFv[i]


            if (with_SSC[SSC]==1):
                Fvm_IC = grb.Fvmax_IC(Fvm, n1, Ri)
    #                Fvt_IC_n = fview_n**3. *grb.FvIC_simp(v/fview_n,gm_m_n, gm_c_n, vm_n, vc_n, Fvm_IC_n, pp_n)  *Funit
    #                Fvt_IC = fview**3. *grb.FvIC(v/fview,gm_a,gm_m, gm_c, va,vm, vc, vMax, Fvm_IC, GM21,pp) *Funit
                Fvt_IC = fviewF *grb.FvIC(v/fview,gm_a,gm_m, gm_c, va,vm, vc, vMax, Fvm_IC, GM21,pp) *Funit
                lgFv_SSC[l][i] = np.log10(Fvt_IC)
    #                LC[i,2]=lgFv_SSC[i]


# #---------------Table for critical time
#         if ((t_dec <= to[i]/Tunits[time_unit]) and (t_dec > to[i-1]/Tunits[time_unit])):
#             for j in range(0,inump):
#                 lgtdec[j]=np.log10(t_dec)
#                 lgFluxdec[j]=lgFv[i]-j/2.
# #                    lgFluxdec_n[j]=np.log10(fview_n**3. *Fvm_n)-j/2.
#         if ((t_Sed <= to[i]/Tunits[time_unit]) and (t_Sed > to[i-1]/Tunits[time_unit])):
#             for j in range(0,inump):
#                 lgtSed[j]=np.log10(t_Sed)
#                 lgFluxSed[j]=lgFv[i]-j/2.

# #---------------Spectrum----------------------------
# #            mytime=Time[self.Time_mode]
#         if( (to[i]/cgs.day) >= mytime) and ( (to[i-1]/cgs.day) < mytime):
#             nu_a=va/Nuunits[nu_unit] *fview
#             nu_min=vm/Nuunits[nu_unit] *fview
#             nu_cool=vc/Nuunits[nu_unit] *fview
# #---------------Spectral index
#             dv = 1.05
# #                Fvtip = fview**3. *grb.Fv(dv* v/fview,va,vm,vc,Fvm,pp)  *Funit
#             if (Smooth=='Yes'):
#                 Fvtip=fviewF *grb.Fv(dv* v/fview,va,vm,vc,Fvm,pp) *Funit
#             else:
#                 Fvtip=fviewF *grb.Fv_sbpl(dv* v/fview,va,vm,vc,Fvm,pp) *Funit

# #                dF_n = grb.Fv(1.01* v/fview_n,va_n,vm_n,vc_n,Fvm_n,pp_n)/grb.Fv(0.99* v/fview_n,va_n,vm_n,vc_n,Fvm_n,pp_n)
# #                dv_n = 1.01/0.99
#             dF = Fvtip/Fvti
#             lgdF = math.log(dF,10)
#             lgdv = math.log(dv,10)
# #                print("###[Spec Index]:",lgdF/lgdv)

# #---------------Table for critical syncrotron frequecies
# #                Fvva=fview**3. *grb.Fv(va,va,vm,vc,Fvm,pp)
# #                Fvvm=fview**3. *grb.Fv(vm,va,vm,vc,Fvm,pp)
# #                Fvvc=fview**3. *grb.Fv(vc,va,vm,vc,Fvm,pp)
#             if (Smooth=='Yes'):
#                 Fvva=fviewF *grb.Fv(va,va,vm,vc,Fvm,pp)
#                 Fvvm=fviewF *grb.Fv(vm,va,vm,vc,Fvm,pp)
#                 Fvvc=fviewF *grb.Fv(vc,va,vm,vc,Fvm,pp)
#             else:
#                 Fvva=fviewF *grb.Fv_sbpl(va,va,vm,vc,Fvm,pp)
#                 Fvvm=fviewF *grb.Fv_sbpl(vm,va,vm,vc,Fvm,pp)
#                 Fvvc=fviewF *grb.Fv_sbpl(vc,va,vm,vc,Fvm,pp)
                
#             for j in range(0,inump):
#                 lgNua[j]=np.log10(nu_a)
#                 lgNum[j]=np.log10(nu_min)
#                 lgNuc[j]=np.log10(nu_cool)
#                 lgFluxa[j]=np.log10(Fvva)-j/2.
#                 lgFluxm[j]=np.log10(Fvvm)-j/2.
#                 lgFluxc[j]=np.log10(Fvvc)-j/2.

#             for j in range(1, inumv):
#                 vj=lgNu[j]
# #                    Fvtj=fview**3. *grb.Fv(vj/fview,va,vm,vc,Fvm,pp)
#                 if (Smooth=='Yes'):
#                     Fvtj=fviewF *grb.Fv(vj/fview,va,vm,vc,Fvm,pp)
#                 else:
#                     Fvtj=fviewF *grb.Fv_sbpl(vj/fview,va,vm,vc,Fvm,pp)
#                 if (vj/fview>=vMax):
#                     if (vj/fview/vMax > 300.):
#                         Fvtj=Fvtj*np.exp(-300.+1.)
#                     else:
#                         Fvtj=Fvtj*np.exp(-vj/fview/vMax+1.)
# #                    vmax=grb.nu_Max(GM21,zi)
# #                    Fvtj=grb.Fv_simp(vj,vm,vc,vmax, Fvm,pp)
#                 lgNu[j]=np.log10(vj/Nuunits[nu_unit])
#                 lgFnu[j]=np.log10(Fvtj)
# #                    lgvFluxv[j]=np.log10(vj*Fvtj)
# #                    Fvv[j,0]=lgNuv[j]
# #                    Fvv[j,1]=lgFluxv[j]
# #                    Spec[j,0]=lgNuv[j]
# #                    Spec[j,1]=lgFluxv[j]
# #                    Spec[j,2]=lgvFluxv[j]

#                 if (with_SSC[SSC]==1):
# #                        Fvtj_IC= fview_n**3. *grb.FvIC_simp(vj_n/fview_n,gm_m_n, gm_c_n, vm_n, vc_n, Fvm_IC_n, pp_n)
# #                        Fvtj_IC =fview**3. *grb.FvIC(vj/fview,gm_a, gm_m, gm_c,va,vm, vc, vMax, Fvm_IC, GM21, pp)
#                     Fvtj_IC =fviewF *grb.FvIC(vj/fview,gm_a, gm_m, gm_c,va,vm, vc, vMax, Fvm_IC, GM21, pp)
#                     lgFnu_SSC[j]=np.log10(Fvtj_IC)
# #                        lgvFluxv_SSC[j]=np.log10(vj*Fvtj_IC)
# #                        Spec[j,3]=lgFluxv_SSC[j]
# #                        Spec[j,4]=lgvFluxv_SSC[j]


#---------------Parameters-----------
        para[i,0]=ti/Tunits[time_unit]
        para[i,1]=Ri
        para[i,2]=GM21
        para[i,3]=beta
        para[i,4]=n2
        para[i,5]=ei
        para[i,6]=Bc
        para[i,7]=Uph
        para[i,8]=UB
        para[i,9]=gm_a
        para[i,10]=gm_m
        para[i,11]=gm_c
        para[i,12]=gm_Max
        para[i,13]=va*fview
        para[i,14]=vm*fview
        para[i,15]=vc*fview
        para[i,16]=vMax*fview
        para[i,17]=Fvm
        para[i,18]=Fvm_IC


#-----------small correction
    Gmt[0]=Gmt[1]
    lgt[0]=lgt[2]
    lgFv[:,0]=lgFv[:,2]
    # lgNu[0]=lgNu[2]
    # lgFnu[0]=lgFnu[2]
    lgt[1]=lgt[2]
    lgFv[:,1]=lgFv[:,2]
    # lgNu[1]=lgNu[2]
    # lgFnu[1]=lgFnu[2]
    lgFv_SSC[:,0]=lgFv_SSC[:,2]
    # lgFnu_SSC[0]=lgFnu_SSC[2]
    lgFv_SSC[:,1]=lgFv_SSC[:,2]
    # lgFnu_SSC[1]=lgFnu_SSC[2]
    for j in range(0,19):
        para[0,j]=para[1,j]

#-------transfer data to global
    paralist=para
    # lgLC=np.column_stack((lgt,lgFv.T))
    # lgSpec=np.column_stack((lgNu,lgFnu))
    # lgLC_SSC=np.column_stack((lgt,lgFv_SSC.T))
    # lgSpec_SSC=np.column_stack((lgNu,lgFnu_SSC))

#================end of Forward shock==================
    if SSC=='Yes':
        lgFv = np.log10(np.power(10.0,lgFv-3.0)+np.power(10.0,lgFv_SSC-3.0))       # convert unit to "mJy"
    else:
        lgFv = lgFv-3.0   # convert unit to "mJy"

#--------Estimate the flux of obs_time (add by fsy)
    estimate_model_log_flux = np.zeros(len(log_obs_time))
    for i,t in enumerate(log_obs_time):
        for iline in range(len(vlist)):
            if nu_obs[i] == vlist[iline]:
                vline = iline
                break
        model_log_time = lgt
        estimate_model_log_flux[i] = np.interp(t,model_log_time,lgFv[vline],left=-99.0,right=99.0)
    
    return 10**estimate_model_log_flux


